MV_FLAG = 4096  # Multi-value flag

PT_UNSPECIFIED = 0
PT_NULL = 1
PT_I2 = 2
PT_LONG = 3
PT_R4 = 4
PT_DOUBLE = 5
PT_CURRENCY = 6
PT_APPTIME = 7
PT_ERROR = 10
PT_BOOLEAN = 11
PT_OBJECT = 13
PT_I8 = 20
PT_STRING8 = 30
PT_UNICODE = 31
PT_SYSTIME = 64
PT_CLSID = 72
PT_BINARY = 258

PT_SHORT = PT_I2
PT_I4 = PT_LONG
PT_FLOAT = PT_R4
PT_R8 = PT_DOUBLE
PT_LONGLONG = PT_I8

PT_MV_I2 = MV_FLAG | PT_I2
PT_MV_LONG = MV_FLAG | PT_LONG
PT_MV_R4 = MV_FLAG | PT_R4
PT_MV_DOUBLE = MV_FLAG | PT_DOUBLE
PT_MV_CURRENCY = MV_FLAG | PT_CURRENCY
PT_MV_APPTIME = MV_FLAG | PT_APPTIME
PT_MV_SYSTIME = MV_FLAG | PT_SYSTIME
PT_MV_STRING8 = MV_FLAG | PT_STRING8
PT_MV_BINARY = MV_FLAG | PT_BINARY
PT_MV_UNICODE = MV_FLAG | PT_UNICODE
PT_MV_CLSID = MV_FLAG | PT_CLSID
PT_MV_I8 = MV_FLAG | PT_I8

PT_MV_SHORT = PT_MV_I2
PT_MV_I4 = PT_MV_LONG
PT_MV_FLOAT = PT_MV_R4
PT_MV_R8 = PT_MV_DOUBLE
PT_MV_LONGLONG = PT_MV_I8

PT_TSTRING = PT_UNICODE  # ???
PT_MV_TSTRING = MV_FLAG | PT_UNICODE


PROP_TYPE_MASK = 65535  # Mask for Property type


def PROP_TYPE(ulPropTag):
    return ulPropTag & PROP_TYPE_MASK


def PROP_ID(ulPropTag):
    return ulPropTag >> 16


def PROP_TAG(ulPropType, ulPropID):
    return (ulPropID << 16) | (ulPropType)


PROP_ID_NULL = 0
PROP_ID_INVALID = 65535
PR_NULL = PROP_TAG(PT_NULL, PROP_ID_NULL)


PR_ACKNOWLEDGEMENT_MODE = PROP_TAG(PT_LONG, 1)
PR_ACKNOWLEDGEMENT_MODE = PROP_TAG(PT_LONG, 1)
PR_ALTERNATE_RECIPIENT_ALLOWED = PROP_TAG(PT_BOOLEAN, 2)
PR_AUTHORIZING_USERS = PROP_TAG(PT_BINARY, 3)
PR_AUTO_FORWARD_COMMENT = PROP_TAG(PT_TSTRING, 4)
PR_AUTO_FORWARD_COMMENT_W = PROP_TAG(PT_UNICODE, 4)
PR_AUTO_FORWARD_COMMENT_W = PROP_TAG(PT_UNICODE, 4)
PR_AUTO_FORWARD_COMMENT_A = PROP_TAG(PT_STRING8, 4)
PR_AUTO_FORWARDED = PROP_TAG(PT_BOOLEAN, 5)
PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID = PROP_TAG(PT_BINARY, 6)
PR_CONTENT_CORRELATOR = PROP_TAG(PT_BINARY, 7)
PR_CONTENT_IDENTIFIER = PROP_TAG(PT_TSTRING, 8)
PR_CONTENT_IDENTIFIER_W = PROP_TAG(PT_UNICODE, 8)
PR_CONTENT_IDENTIFIER_A = PROP_TAG(PT_STRING8, 8)
PR_CONTENT_LENGTH = PROP_TAG(PT_LONG, 9)
PR_CONTENT_RETURN_REQUESTED = PROP_TAG(PT_BOOLEAN, 10)
PR_CONVERSATION_KEY = PROP_TAG(PT_BINARY, 11)
PR_CONVERSION_EITS = PROP_TAG(PT_BINARY, 12)
PR_CONVERSION_WITH_LOSS_PROHIBITED = PROP_TAG(PT_BOOLEAN, 13)
PR_CONVERTED_EITS = PROP_TAG(PT_BINARY, 14)
PR_DEFERRED_DELIVERY_TIME = PROP_TAG(PT_SYSTIME, 15)
PR_DELIVER_TIME = PROP_TAG(PT_SYSTIME, 16)
PR_DISCARD_REASON = PROP_TAG(PT_LONG, 17)
PR_DISCLOSURE_OF_RECIPIENTS = PROP_TAG(PT_BOOLEAN, 18)
PR_DL_EXPANSION_HISTORY = PROP_TAG(PT_BINARY, 19)
PR_DL_EXPANSION_PROHIBITED = PROP_TAG(PT_BOOLEAN, 20)
PR_EXPIRY_TIME = PROP_TAG(PT_SYSTIME, 21)
PR_IMPLICIT_CONVERSION_PROHIBITED = PROP_TAG(PT_BOOLEAN, 22)
PR_IMPORTANCE = PROP_TAG(PT_LONG, 23)
PR_IPM_ID = PROP_TAG(PT_BINARY, 24)
PR_LATEST_DELIVERY_TIME = PROP_TAG(PT_SYSTIME, 25)
PR_MESSAGE_CLASS = PROP_TAG(PT_TSTRING, 26)
PR_MESSAGE_CLASS_W = PROP_TAG(PT_UNICODE, 26)
PR_MESSAGE_CLASS_A = PROP_TAG(PT_STRING8, 26)
PR_MESSAGE_DELIVERY_ID = PROP_TAG(PT_BINARY, 27)
PR_MESSAGE_SECURITY_LABEL = PROP_TAG(PT_BINARY, 30)
PR_OBSOLETED_IPMS = PROP_TAG(PT_BINARY, 31)
PR_ORIGINALLY_INTENDED_RECIPIENT_NAME = PROP_TAG(PT_BINARY, 32)
PR_ORIGINAL_EITS = PROP_TAG(PT_BINARY, 33)
PR_ORIGINATOR_CERTIFICATE = PROP_TAG(PT_BINARY, 34)
PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED = PROP_TAG(PT_BOOLEAN, 35)
PR_ORIGINATOR_RETURN_ADDRESS = PROP_TAG(PT_BINARY, 36)
PR_PARENT_KEY = PROP_TAG(PT_BINARY, 37)
PR_PRIORITY = PROP_TAG(PT_LONG, 38)
PR_ORIGIN_CHECK = PROP_TAG(PT_BINARY, 39)
PR_PROOF_OF_SUBMISSION_REQUESTED = PROP_TAG(PT_BOOLEAN, 40)
PR_READ_RECEIPT_REQUESTED = PROP_TAG(PT_BOOLEAN, 41)
PR_RECEIPT_TIME = PROP_TAG(PT_SYSTIME, 42)
PR_RECIPIENT_REASSIGNMENT_PROHIBITED = PROP_TAG(PT_BOOLEAN, 43)
PR_REDIRECTION_HISTORY = PROP_TAG(PT_BINARY, 44)
PR_RELATED_IPMS = PROP_TAG(PT_BINARY, 45)
PR_ORIGINAL_SENSITIVITY = PROP_TAG(PT_LONG, 46)
PR_LANGUAGES = PROP_TAG(PT_TSTRING, 47)
PR_LANGUAGES_W = PROP_TAG(PT_UNICODE, 47)
PR_LANGUAGES_A = PROP_TAG(PT_STRING8, 47)
PR_REPLY_TIME = PROP_TAG(PT_SYSTIME, 48)
PR_REPORT_TAG = PROP_TAG(PT_BINARY, 49)
PR_REPORT_TIME = PROP_TAG(PT_SYSTIME, 50)
PR_RETURNED_IPM = PROP_TAG(PT_BOOLEAN, 51)
PR_SECURITY = PROP_TAG(PT_LONG, 52)
PR_INCOMPLETE_COPY = PROP_TAG(PT_BOOLEAN, 53)
PR_SENSITIVITY = PROP_TAG(PT_LONG, 54)
PR_SUBJECT = PROP_TAG(PT_TSTRING, 55)
PR_SUBJECT_W = PROP_TAG(PT_UNICODE, 55)
PR_SUBJECT_A = PROP_TAG(PT_STRING8, 55)
PR_SUBJECT_IPM = PROP_TAG(PT_BINARY, 56)
PR_CLIENT_SUBMIT_TIME = PROP_TAG(PT_SYSTIME, 57)
PR_REPORT_NAME = PROP_TAG(PT_TSTRING, 58)
PR_REPORT_NAME_W = PROP_TAG(PT_UNICODE, 58)
PR_REPORT_NAME_A = PROP_TAG(PT_STRING8, 58)
PR_SENT_REPRESENTING_SEARCH_KEY = PROP_TAG(PT_BINARY, 59)
PR_X400_CONTENT_TYPE = PROP_TAG(PT_BINARY, 60)
PR_SUBJECT_PREFIX = PROP_TAG(PT_TSTRING, 61)
PR_SUBJECT_PREFIX_W = PROP_TAG(PT_UNICODE, 61)
PR_SUBJECT_PREFIX_A = PROP_TAG(PT_STRING8, 61)
PR_NON_RECEIPT_REASON = PROP_TAG(PT_LONG, 62)
PR_RECEIVED_BY_ENTRYID = PROP_TAG(PT_BINARY, 63)
PR_RECEIVED_BY_NAME = PROP_TAG(PT_TSTRING, 64)
PR_RECEIVED_BY_NAME_W = PROP_TAG(PT_UNICODE, 64)
PR_RECEIVED_BY_NAME_A = PROP_TAG(PT_STRING8, 64)
PR_SENT_REPRESENTING_ENTRYID = PROP_TAG(PT_BINARY, 65)
PR_SENT_REPRESENTING_NAME = PROP_TAG(PT_TSTRING, 66)
PR_SENT_REPRESENTING_NAME_W = PROP_TAG(PT_UNICODE, 66)
PR_SENT_REPRESENTING_NAME_A = PROP_TAG(PT_STRING8, 66)
PR_RCVD_REPRESENTING_ENTRYID = PROP_TAG(PT_BINARY, 67)
PR_RCVD_REPRESENTING_NAME = PROP_TAG(PT_TSTRING, 68)
PR_RCVD_REPRESENTING_NAME_W = PROP_TAG(PT_UNICODE, 68)
PR_RCVD_REPRESENTING_NAME_A = PROP_TAG(PT_STRING8, 68)
PR_REPORT_ENTRYID = PROP_TAG(PT_BINARY, 69)
PR_READ_RECEIPT_ENTRYID = PROP_TAG(PT_BINARY, 70)
PR_MESSAGE_SUBMISSION_ID = PROP_TAG(PT_BINARY, 71)
PR_PROVIDER_SUBMIT_TIME = PROP_TAG(PT_SYSTIME, 72)
PR_ORIGINAL_SUBJECT = PROP_TAG(PT_TSTRING, 73)
PR_ORIGINAL_SUBJECT_W = PROP_TAG(PT_UNICODE, 73)
PR_ORIGINAL_SUBJECT_A = PROP_TAG(PT_STRING8, 73)
PR_DISC_VAL = PROP_TAG(PT_BOOLEAN, 74)
PR_ORIG_MESSAGE_CLASS = PROP_TAG(PT_TSTRING, 75)
PR_ORIG_MESSAGE_CLASS_W = PROP_TAG(PT_UNICODE, 75)
PR_ORIG_MESSAGE_CLASS_A = PROP_TAG(PT_STRING8, 75)
PR_ORIGINAL_AUTHOR_ENTRYID = PROP_TAG(PT_BINARY, 76)
PR_ORIGINAL_AUTHOR_NAME = PROP_TAG(PT_TSTRING, 77)
PR_ORIGINAL_AUTHOR_NAME_W = PROP_TAG(PT_UNICODE, 77)
PR_ORIGINAL_AUTHOR_NAME_A = PROP_TAG(PT_STRING8, 77)
PR_ORIGINAL_SUBMIT_TIME = PROP_TAG(PT_SYSTIME, 78)
PR_REPLY_RECIPIENT_ENTRIES = PROP_TAG(PT_BINARY, 79)
PR_REPLY_RECIPIENT_NAMES = PROP_TAG(PT_TSTRING, 80)
PR_REPLY_RECIPIENT_NAMES_W = PROP_TAG(PT_UNICODE, 80)
PR_REPLY_RECIPIENT_NAMES_A = PROP_TAG(PT_STRING8, 80)
PR_RECEIVED_BY_SEARCH_KEY = PROP_TAG(PT_BINARY, 81)
PR_RCVD_REPRESENTING_SEARCH_KEY = PROP_TAG(PT_BINARY, 82)
PR_READ_RECEIPT_SEARCH_KEY = PROP_TAG(PT_BINARY, 83)
PR_REPORT_SEARCH_KEY = PROP_TAG(PT_BINARY, 84)
PR_ORIGINAL_DELIVERY_TIME = PROP_TAG(PT_SYSTIME, 85)
PR_ORIGINAL_AUTHOR_SEARCH_KEY = PROP_TAG(PT_BINARY, 86)
PR_MESSAGE_TO_ME = PROP_TAG(PT_BOOLEAN, 87)
PR_MESSAGE_CC_ME = PROP_TAG(PT_BOOLEAN, 88)
PR_MESSAGE_RECIP_ME = PROP_TAG(PT_BOOLEAN, 89)
PR_ORIGINAL_SENDER_NAME = PROP_TAG(PT_TSTRING, 90)
PR_ORIGINAL_SENDER_NAME_W = PROP_TAG(PT_UNICODE, 90)
PR_ORIGINAL_SENDER_NAME_A = PROP_TAG(PT_STRING8, 90)
PR_ORIGINAL_SENDER_ENTRYID = PROP_TAG(PT_BINARY, 91)
PR_ORIGINAL_SENDER_SEARCH_KEY = PROP_TAG(PT_BINARY, 92)
PR_ORIGINAL_SENT_REPRESENTING_NAME = PROP_TAG(PT_TSTRING, 93)
PR_ORIGINAL_SENT_REPRESENTING_NAME_W = PROP_TAG(PT_UNICODE, 93)
PR_ORIGINAL_SENT_REPRESENTING_NAME_A = PROP_TAG(PT_STRING8, 93)
PR_ORIGINAL_SENT_REPRESENTING_ENTRYID = PROP_TAG(PT_BINARY, 94)
PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY = PROP_TAG(PT_BINARY, 95)
PR_START_DATE = PROP_TAG(PT_SYSTIME, 96)
PR_END_DATE = PROP_TAG(PT_SYSTIME, 97)
PR_OWNER_APPT_ID = PROP_TAG(PT_LONG, 98)
PR_RESPONSE_REQUESTED = PROP_TAG(PT_BOOLEAN, 99)
PR_SENT_REPRESENTING_ADDRTYPE = PROP_TAG(PT_TSTRING, 100)
PR_SENT_REPRESENTING_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 100)
PR_SENT_REPRESENTING_ADDRTYPE_A = PROP_TAG(PT_STRING8, 100)
PR_SENT_REPRESENTING_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 101)
PR_SENT_REPRESENTING_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 101)
PR_SENT_REPRESENTING_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 101)
PR_ORIGINAL_SENDER_ADDRTYPE = PROP_TAG(PT_TSTRING, 102)
PR_ORIGINAL_SENDER_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 102)
PR_ORIGINAL_SENDER_ADDRTYPE_A = PROP_TAG(PT_STRING8, 102)
PR_ORIGINAL_SENDER_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 103)
PR_ORIGINAL_SENDER_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 103)
PR_ORIGINAL_SENDER_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 103)
PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE = PROP_TAG(PT_TSTRING, 104)
PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 104)
PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_A = PROP_TAG(PT_STRING8, 104)
PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 105)
PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 105)
PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 105)
PR_CONVERSATION_TOPIC = PROP_TAG(PT_TSTRING, 112)
PR_CONVERSATION_TOPIC_W = PROP_TAG(PT_UNICODE, 112)
PR_CONVERSATION_TOPIC_A = PROP_TAG(PT_STRING8, 112)
PR_CONVERSATION_INDEX = PROP_TAG(PT_BINARY, 113)
PR_ORIGINAL_DISPLAY_BCC = PROP_TAG(PT_TSTRING, 114)
PR_ORIGINAL_DISPLAY_BCC_W = PROP_TAG(PT_UNICODE, 114)
PR_ORIGINAL_DISPLAY_BCC_A = PROP_TAG(PT_STRING8, 114)
PR_ORIGINAL_DISPLAY_CC = PROP_TAG(PT_TSTRING, 115)
PR_ORIGINAL_DISPLAY_CC_W = PROP_TAG(PT_UNICODE, 115)
PR_ORIGINAL_DISPLAY_CC_A = PROP_TAG(PT_STRING8, 115)
PR_ORIGINAL_DISPLAY_TO = PROP_TAG(PT_TSTRING, 116)
PR_ORIGINAL_DISPLAY_TO_W = PROP_TAG(PT_UNICODE, 116)
PR_ORIGINAL_DISPLAY_TO_A = PROP_TAG(PT_STRING8, 116)
PR_RECEIVED_BY_ADDRTYPE = PROP_TAG(PT_TSTRING, 117)
PR_RECEIVED_BY_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 117)
PR_RECEIVED_BY_ADDRTYPE_A = PROP_TAG(PT_STRING8, 117)
PR_RECEIVED_BY_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 118)
PR_RECEIVED_BY_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 118)
PR_RECEIVED_BY_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 118)
PR_RCVD_REPRESENTING_ADDRTYPE = PROP_TAG(PT_TSTRING, 119)
PR_RCVD_REPRESENTING_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 119)
PR_RCVD_REPRESENTING_ADDRTYPE_A = PROP_TAG(PT_STRING8, 119)
PR_RCVD_REPRESENTING_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 120)
PR_RCVD_REPRESENTING_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 120)
PR_RCVD_REPRESENTING_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 120)
PR_ORIGINAL_AUTHOR_ADDRTYPE = PROP_TAG(PT_TSTRING, 121)
PR_ORIGINAL_AUTHOR_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 121)
PR_ORIGINAL_AUTHOR_ADDRTYPE_A = PROP_TAG(PT_STRING8, 121)
PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 122)
PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 122)
PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 122)
PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE = PROP_TAG(PT_TSTRING, 123)
PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 123)
PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_A = PROP_TAG(PT_STRING8, 123)
PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 124)
PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 124)
PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 124)
PR_TRANSPORT_MESSAGE_HEADERS = PROP_TAG(PT_TSTRING, 125)
PR_TRANSPORT_MESSAGE_HEADERS_W = PROP_TAG(PT_UNICODE, 125)
PR_TRANSPORT_MESSAGE_HEADERS_A = PROP_TAG(PT_STRING8, 125)
PR_DELEGATION = PROP_TAG(PT_BINARY, 126)
PR_TNEF_CORRELATION_KEY = PROP_TAG(PT_BINARY, 127)
PR_BODY = PROP_TAG(PT_TSTRING, 4096)
PR_BODY_W = PROP_TAG(PT_UNICODE, 4096)
PR_BODY_A = PROP_TAG(PT_STRING8, 4096)
PR_BODY_HTML = PROP_TAG(PT_TSTRING, 4115)
PR_BODY_HTML_W = PROP_TAG(PT_UNICODE, 4115)
PR_BODY_HTML_A = PROP_TAG(PT_STRING8, 4115)
PR_REPORT_TEXT = PROP_TAG(PT_TSTRING, 4097)
PR_REPORT_TEXT_W = PROP_TAG(PT_UNICODE, 4097)
PR_REPORT_TEXT_A = PROP_TAG(PT_STRING8, 4097)
PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY = PROP_TAG(PT_BINARY, 4098)
PR_REPORTING_DL_NAME = PROP_TAG(PT_BINARY, 4099)
PR_REPORTING_MTA_CERTIFICATE = PROP_TAG(PT_BINARY, 4100)
PR_RTF_SYNC_BODY_CRC = PROP_TAG(PT_LONG, 4102)
PR_RTF_SYNC_BODY_COUNT = PROP_TAG(PT_LONG, 4103)
PR_RTF_SYNC_BODY_TAG = PROP_TAG(PT_TSTRING, 4104)
PR_RTF_SYNC_BODY_TAG_W = PROP_TAG(PT_UNICODE, 4104)
PR_RTF_SYNC_BODY_TAG_A = PROP_TAG(PT_STRING8, 4104)
PR_RTF_COMPRESSED = PROP_TAG(PT_BINARY, 4105)
PR_RTF_SYNC_PREFIX_COUNT = PROP_TAG(PT_LONG, 4112)
PR_RTF_SYNC_TRAILING_COUNT = PROP_TAG(PT_LONG, 4113)
PR_ORIGINALLY_INTENDED_RECIP_ENTRYID = PROP_TAG(PT_BINARY, 4114)
PR_CONTENT_INTEGRITY_CHECK = PROP_TAG(PT_BINARY, 3072)
PR_EXPLICIT_CONVERSION = PROP_TAG(PT_LONG, 3073)
PR_IPM_RETURN_REQUESTED = PROP_TAG(PT_BOOLEAN, 3074)
PR_MESSAGE_TOKEN = PROP_TAG(PT_BINARY, 3075)
PR_NDR_REASON_CODE = PROP_TAG(PT_LONG, 3076)
PR_NDR_DIAG_CODE = PROP_TAG(PT_LONG, 3077)
PR_NON_RECEIPT_NOTIFICATION_REQUESTED = PROP_TAG(PT_BOOLEAN, 3078)
PR_DELIVERY_POINT = PROP_TAG(PT_LONG, 3079)
PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED = PROP_TAG(PT_BOOLEAN, 3080)
PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT = PROP_TAG(PT_BINARY, 3081)
PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY = PROP_TAG(PT_BOOLEAN, 3082)
PR_PHYSICAL_DELIVERY_MODE = PROP_TAG(PT_LONG, 3083)
PR_PHYSICAL_DELIVERY_REPORT_REQUEST = PROP_TAG(PT_LONG, 3084)
PR_PHYSICAL_FORWARDING_ADDRESS = PROP_TAG(PT_BINARY, 3085)
PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED = PROP_TAG(PT_BOOLEAN, 3086)
PR_PHYSICAL_FORWARDING_PROHIBITED = PROP_TAG(PT_BOOLEAN, 3087)
PR_PHYSICAL_RENDITION_ATTRIBUTES = PROP_TAG(PT_BINARY, 3088)
PR_PROOF_OF_DELIVERY = PROP_TAG(PT_BINARY, 3089)
PR_PROOF_OF_DELIVERY_REQUESTED = PROP_TAG(PT_BOOLEAN, 3090)
PR_RECIPIENT_CERTIFICATE = PROP_TAG(PT_BINARY, 3091)
PR_RECIPIENT_NUMBER_FOR_ADVICE = PROP_TAG(PT_TSTRING, 3092)
PR_RECIPIENT_NUMBER_FOR_ADVICE_W = PROP_TAG(PT_UNICODE, 3092)
PR_RECIPIENT_NUMBER_FOR_ADVICE_A = PROP_TAG(PT_STRING8, 3092)
PR_RECIPIENT_TYPE = PROP_TAG(PT_LONG, 3093)
PR_REGISTERED_MAIL_TYPE = PROP_TAG(PT_LONG, 3094)
PR_REPLY_REQUESTED = PROP_TAG(PT_BOOLEAN, 3095)
PR_REQUESTED_DELIVERY_METHOD = PROP_TAG(PT_LONG, 3096)
PR_SENDER_ENTRYID = PROP_TAG(PT_BINARY, 3097)
PR_SENDER_NAME = PROP_TAG(PT_TSTRING, 3098)
PR_SENDER_NAME_W = PROP_TAG(PT_UNICODE, 3098)
PR_SENDER_NAME_A = PROP_TAG(PT_STRING8, 3098)
PR_SUPPLEMENTARY_INFO = PROP_TAG(PT_TSTRING, 3099)
PR_SUPPLEMENTARY_INFO_W = PROP_TAG(PT_UNICODE, 3099)
PR_SUPPLEMENTARY_INFO_A = PROP_TAG(PT_STRING8, 3099)
PR_TYPE_OF_MTS_USER = PROP_TAG(PT_LONG, 3100)
PR_SENDER_SEARCH_KEY = PROP_TAG(PT_BINARY, 3101)
PR_SENDER_ADDRTYPE = PROP_TAG(PT_TSTRING, 3102)
PR_SENDER_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 3102)
PR_SENDER_ADDRTYPE_A = PROP_TAG(PT_STRING8, 3102)
PR_SENDER_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 3103)
PR_SENDER_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 3103)
PR_SENDER_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 3103)
PR_CURRENT_VERSION = PROP_TAG(PT_I8, 3584)
PR_DELETE_AFTER_SUBMIT = PROP_TAG(PT_BOOLEAN, 3585)
PR_DISPLAY_BCC = PROP_TAG(PT_TSTRING, 3586)
PR_DISPLAY_BCC_W = PROP_TAG(PT_UNICODE, 3586)
PR_DISPLAY_BCC_A = PROP_TAG(PT_STRING8, 3586)
PR_DISPLAY_CC = PROP_TAG(PT_TSTRING, 3587)
PR_DISPLAY_CC_W = PROP_TAG(PT_UNICODE, 3587)
PR_DISPLAY_CC_A = PROP_TAG(PT_STRING8, 3587)
PR_DISPLAY_TO = PROP_TAG(PT_TSTRING, 3588)
PR_DISPLAY_TO_W = PROP_TAG(PT_UNICODE, 3588)
PR_DISPLAY_TO_A = PROP_TAG(PT_STRING8, 3588)
PR_PARENT_DISPLAY = PROP_TAG(PT_TSTRING, 3589)
PR_PARENT_DISPLAY_W = PROP_TAG(PT_UNICODE, 3589)
PR_PARENT_DISPLAY_A = PROP_TAG(PT_STRING8, 3589)
PR_MESSAGE_DELIVERY_TIME = PROP_TAG(PT_SYSTIME, 3590)
PR_MESSAGE_FLAGS = PROP_TAG(PT_LONG, 3591)
PR_MESSAGE_SIZE = PROP_TAG(PT_LONG, 3592)
PR_PARENT_ENTRYID = PROP_TAG(PT_BINARY, 3593)
PR_SENTMAIL_ENTRYID = PROP_TAG(PT_BINARY, 3594)
PR_CORRELATE = PROP_TAG(PT_BOOLEAN, 3596)
PR_CORRELATE_MTSID = PROP_TAG(PT_BINARY, 3597)
PR_DISCRETE_VALUES = PROP_TAG(PT_BOOLEAN, 3598)
PR_RESPONSIBILITY = PROP_TAG(PT_BOOLEAN, 3599)
PR_SPOOLER_STATUS = PROP_TAG(PT_LONG, 3600)
PR_TRANSPORT_STATUS = PROP_TAG(PT_LONG, 3601)
PR_MESSAGE_RECIPIENTS = PROP_TAG(PT_OBJECT, 3602)
PR_MESSAGE_ATTACHMENTS = PROP_TAG(PT_OBJECT, 3603)
PR_SUBMIT_FLAGS = PROP_TAG(PT_LONG, 3604)
PR_RECIPIENT_STATUS = PROP_TAG(PT_LONG, 3605)
PR_TRANSPORT_KEY = PROP_TAG(PT_LONG, 3606)
PR_MSG_STATUS = PROP_TAG(PT_LONG, 3607)
PR_MESSAGE_DOWNLOAD_TIME = PROP_TAG(PT_LONG, 3608)
PR_CREATION_VERSION = PROP_TAG(PT_I8, 3609)
PR_MODIFY_VERSION = PROP_TAG(PT_I8, 3610)
PR_HASATTACH = PROP_TAG(PT_BOOLEAN, 3611)
PR_BODY_CRC = PROP_TAG(PT_LONG, 3612)
PR_NORMALIZED_SUBJECT = PROP_TAG(PT_TSTRING, 3613)
PR_NORMALIZED_SUBJECT_W = PROP_TAG(PT_UNICODE, 3613)
PR_NORMALIZED_SUBJECT_A = PROP_TAG(PT_STRING8, 3613)
PR_RTF_IN_SYNC = PROP_TAG(PT_BOOLEAN, 3615)
PR_ATTACH_SIZE = PROP_TAG(PT_LONG, 3616)
PR_ATTACH_NUM = PROP_TAG(PT_LONG, 3617)
PR_PREPROCESS = PROP_TAG(PT_BOOLEAN, 3618)
PR_ORIGINATING_MTA_CERTIFICATE = PROP_TAG(PT_BINARY, 3621)
PR_PROOF_OF_SUBMISSION = PROP_TAG(PT_BINARY, 3622)
PR_ENTRYID = PROP_TAG(PT_BINARY, 4095)
PR_OBJECT_TYPE = PROP_TAG(PT_LONG, 4094)
PR_ICON = PROP_TAG(PT_BINARY, 4093)
PR_MINI_ICON = PROP_TAG(PT_BINARY, 4092)
PR_STORE_ENTRYID = PROP_TAG(PT_BINARY, 4091)
PR_STORE_RECORD_KEY = PROP_TAG(PT_BINARY, 4090)
PR_RECORD_KEY = PROP_TAG(PT_BINARY, 4089)
PR_MAPPING_SIGNATURE = PROP_TAG(PT_BINARY, 4088)
PR_ACCESS_LEVEL = PROP_TAG(PT_LONG, 4087)
PR_INSTANCE_KEY = PROP_TAG(PT_BINARY, 4086)
PR_ROW_TYPE = PROP_TAG(PT_LONG, 4085)
PR_ACCESS = PROP_TAG(PT_LONG, 4084)
PR_ROWID = PROP_TAG(PT_LONG, 12288)
PR_DISPLAY_NAME = PROP_TAG(PT_TSTRING, 12289)
PR_DISPLAY_NAME_W = PROP_TAG(PT_UNICODE, 12289)
PR_DISPLAY_NAME_A = PROP_TAG(PT_STRING8, 12289)
PR_ADDRTYPE = PROP_TAG(PT_TSTRING, 12290)
PR_ADDRTYPE_W = PROP_TAG(PT_UNICODE, 12290)
PR_ADDRTYPE_A = PROP_TAG(PT_STRING8, 12290)
PR_EMAIL_ADDRESS = PROP_TAG(PT_TSTRING, 12291)
PR_EMAIL_ADDRESS_W = PROP_TAG(PT_UNICODE, 12291)
PR_EMAIL_ADDRESS_A = PROP_TAG(PT_STRING8, 12291)
PR_COMMENT = PROP_TAG(PT_TSTRING, 12292)
PR_COMMENT_W = PROP_TAG(PT_UNICODE, 12292)
PR_COMMENT_A = PROP_TAG(PT_STRING8, 12292)
PR_DEPTH = PROP_TAG(PT_LONG, 12293)
PR_PROVIDER_DISPLAY = PROP_TAG(PT_TSTRING, 12294)
PR_PROVIDER_DISPLAY_W = PROP_TAG(PT_UNICODE, 12294)
PR_PROVIDER_DISPLAY_A = PROP_TAG(PT_STRING8, 12294)
PR_CREATION_TIME = PROP_TAG(PT_SYSTIME, 12295)
PR_LAST_MODIFICATION_TIME = PROP_TAG(PT_SYSTIME, 12296)
PR_RESOURCE_FLAGS = PROP_TAG(PT_LONG, 12297)
PR_PROVIDER_DLL_NAME = PROP_TAG(PT_TSTRING, 12298)
PR_PROVIDER_DLL_NAME_W = PROP_TAG(PT_UNICODE, 12298)
PR_PROVIDER_DLL_NAME_A = PROP_TAG(PT_STRING8, 12298)
PR_SEARCH_KEY = PROP_TAG(PT_BINARY, 12299)
PR_PROVIDER_UID = PROP_TAG(PT_BINARY, 12300)
PR_PROVIDER_ORDINAL = PROP_TAG(PT_LONG, 12301)
PR_FORM_VERSION = PROP_TAG(PT_TSTRING, 13057)
PR_FORM_VERSION_W = PROP_TAG(PT_UNICODE, 13057)
PR_FORM_VERSION_A = PROP_TAG(PT_STRING8, 13057)
PR_FORM_CLSID = PROP_TAG(PT_CLSID, 13058)
PR_FORM_CONTACT_NAME = PROP_TAG(PT_TSTRING, 13059)
PR_FORM_CONTACT_NAME_W = PROP_TAG(PT_UNICODE, 13059)
PR_FORM_CONTACT_NAME_A = PROP_TAG(PT_STRING8, 13059)
PR_FORM_CATEGORY = PROP_TAG(PT_TSTRING, 13060)
PR_FORM_CATEGORY_W = PROP_TAG(PT_UNICODE, 13060)
PR_FORM_CATEGORY_A = PROP_TAG(PT_STRING8, 13060)
PR_FORM_CATEGORY_SUB = PROP_TAG(PT_TSTRING, 13061)
PR_FORM_CATEGORY_SUB_W = PROP_TAG(PT_UNICODE, 13061)
PR_FORM_CATEGORY_SUB_A = PROP_TAG(PT_STRING8, 13061)
PR_FORM_HOST_MAP = PROP_TAG(PT_MV_LONG, 13062)
PR_FORM_HIDDEN = PROP_TAG(PT_BOOLEAN, 13063)
PR_FORM_DESIGNER_NAME = PROP_TAG(PT_TSTRING, 13064)
PR_FORM_DESIGNER_NAME_W = PROP_TAG(PT_UNICODE, 13064)
PR_FORM_DESIGNER_NAME_A = PROP_TAG(PT_STRING8, 13064)
PR_FORM_DESIGNER_GUID = PROP_TAG(PT_CLSID, 13065)
PR_FORM_MESSAGE_BEHAVIOR = PROP_TAG(PT_LONG, 13066)
PR_DEFAULT_STORE = PROP_TAG(PT_BOOLEAN, 13312)
PR_STORE_SUPPORT_MASK = PROP_TAG(PT_LONG, 13325)
PR_STORE_STATE = PROP_TAG(PT_LONG, 13326)
PR_IPM_SUBTREE_SEARCH_KEY = PROP_TAG(PT_BINARY, 13328)
PR_IPM_OUTBOX_SEARCH_KEY = PROP_TAG(PT_BINARY, 13329)
PR_IPM_WASTEBASKET_SEARCH_KEY = PROP_TAG(PT_BINARY, 13330)
PR_IPM_SENTMAIL_SEARCH_KEY = PROP_TAG(PT_BINARY, 13331)
PR_MDB_PROVIDER = PROP_TAG(PT_BINARY, 13332)
PR_RECEIVE_FOLDER_SETTINGS = PROP_TAG(PT_OBJECT, 13333)
PR_VALID_FOLDER_MASK = PROP_TAG(PT_LONG, 13791)
PR_IPM_SUBTREE_ENTRYID = PROP_TAG(PT_BINARY, 13792)
PR_IPM_OUTBOX_ENTRYID = PROP_TAG(PT_BINARY, 13794)
PR_IPM_WASTEBASKET_ENTRYID = PROP_TAG(PT_BINARY, 13795)
PR_IPM_SENTMAIL_ENTRYID = PROP_TAG(PT_BINARY, 13796)
PR_VIEWS_ENTRYID = PROP_TAG(PT_BINARY, 13797)
PR_COMMON_VIEWS_ENTRYID = PROP_TAG(PT_BINARY, 13798)
PR_FINDER_ENTRYID = PROP_TAG(PT_BINARY, 13799)
PR_CONTAINER_FLAGS = PROP_TAG(PT_LONG, 13824)
PR_FOLDER_TYPE = PROP_TAG(PT_LONG, 13825)
PR_CONTENT_COUNT = PROP_TAG(PT_LONG, 13826)
PR_CONTENT_UNREAD = PROP_TAG(PT_LONG, 13827)
PR_CREATE_TEMPLATES = PROP_TAG(PT_OBJECT, 13828)
PR_DETAILS_TABLE = PROP_TAG(PT_OBJECT, 13829)
PR_SEARCH = PROP_TAG(PT_OBJECT, 13831)
PR_SELECTABLE = PROP_TAG(PT_BOOLEAN, 13833)
PR_SUBFOLDERS = PROP_TAG(PT_BOOLEAN, 13834)
PR_STATUS = PROP_TAG(PT_LONG, 13835)
PR_ANR = PROP_TAG(PT_TSTRING, 13836)
PR_ANR_W = PROP_TAG(PT_UNICODE, 13836)
PR_ANR_A = PROP_TAG(PT_STRING8, 13836)
PR_CONTENTS_SORT_ORDER = PROP_TAG(PT_MV_LONG, 13837)
PR_CONTAINER_HIERARCHY = PROP_TAG(PT_OBJECT, 13838)
PR_CONTAINER_CONTENTS = PROP_TAG(PT_OBJECT, 13839)
PR_FOLDER_ASSOCIATED_CONTENTS = PROP_TAG(PT_OBJECT, 13840)
PR_DEF_CREATE_DL = PROP_TAG(PT_BINARY, 13841)
PR_DEF_CREATE_MAILUSER = PROP_TAG(PT_BINARY, 13842)
PR_CONTAINER_CLASS = PROP_TAG(PT_TSTRING, 13843)
PR_CONTAINER_CLASS_W = PROP_TAG(PT_UNICODE, 13843)
PR_CONTAINER_CLASS_A = PROP_TAG(PT_STRING8, 13843)
PR_CONTAINER_MODIFY_VERSION = PROP_TAG(PT_I8, 13844)
PR_AB_PROVIDER_ID = PROP_TAG(PT_BINARY, 13845)
PR_DEFAULT_VIEW_ENTRYID = PROP_TAG(PT_BINARY, 13846)
PR_ASSOC_CONTENT_COUNT = PROP_TAG(PT_LONG, 13847)
PR_ATTACHMENT_X400_PARAMETERS = PROP_TAG(PT_BINARY, 14080)
PR_ATTACH_DATA_OBJ = PROP_TAG(PT_OBJECT, 14081)
PR_ATTACH_DATA_BIN = PROP_TAG(PT_BINARY, 14081)
PR_ATTACH_ENCODING = PROP_TAG(PT_BINARY, 14082)
PR_ATTACH_EXTENSION = PROP_TAG(PT_TSTRING, 14083)
PR_ATTACH_EXTENSION_W = PROP_TAG(PT_UNICODE, 14083)
PR_ATTACH_EXTENSION_A = PROP_TAG(PT_STRING8, 14083)
PR_ATTACH_FILENAME = PROP_TAG(PT_TSTRING, 14084)
PR_ATTACH_FILENAME_W = PROP_TAG(PT_UNICODE, 14084)
PR_ATTACH_FILENAME_A = PROP_TAG(PT_STRING8, 14084)
PR_ATTACH_METHOD = PROP_TAG(PT_LONG, 14085)
PR_ATTACH_LONG_FILENAME = PROP_TAG(PT_TSTRING, 14087)
PR_ATTACH_LONG_FILENAME_W = PROP_TAG(PT_UNICODE, 14087)
PR_ATTACH_LONG_FILENAME_A = PROP_TAG(PT_STRING8, 14087)
PR_ATTACH_PATHNAME = PROP_TAG(PT_TSTRING, 14088)
PR_ATTACH_PATHNAME_W = PROP_TAG(PT_UNICODE, 14088)
PR_ATTACH_PATHNAME_A = PROP_TAG(PT_STRING8, 14088)
PR_ATTACH_RENDERING = PROP_TAG(PT_BINARY, 14089)
PR_ATTACH_TAG = PROP_TAG(PT_BINARY, 14090)
PR_RENDERING_POSITION = PROP_TAG(PT_LONG, 14091)
PR_ATTACH_TRANSPORT_NAME = PROP_TAG(PT_TSTRING, 14092)
PR_ATTACH_TRANSPORT_NAME_W = PROP_TAG(PT_UNICODE, 14092)
PR_ATTACH_TRANSPORT_NAME_A = PROP_TAG(PT_STRING8, 14092)
PR_ATTACH_LONG_PATHNAME = PROP_TAG(PT_TSTRING, 14093)
PR_ATTACH_LONG_PATHNAME_W = PROP_TAG(PT_UNICODE, 14093)
PR_ATTACH_LONG_PATHNAME_A = PROP_TAG(PT_STRING8, 14093)
PR_ATTACH_MIME_TAG = PROP_TAG(PT_TSTRING, 14094)
PR_ATTACH_MIME_TAG_W = PROP_TAG(PT_UNICODE, 14094)
PR_ATTACH_MIME_TAG_A = PROP_TAG(PT_STRING8, 14094)
PR_ATTACH_ADDITIONAL_INFO = PROP_TAG(PT_BINARY, 14095)
PR_DISPLAY_TYPE = PROP_TAG(PT_LONG, 14592)
PR_TEMPLATEID = PROP_TAG(PT_BINARY, 14594)
PR_PRIMARY_CAPABILITY = PROP_TAG(PT_BINARY, 14596)
PR_7BIT_DISPLAY_NAME = PROP_TAG(PT_STRING8, 14847)
PR_ACCOUNT = PROP_TAG(PT_TSTRING, 14848)
PR_ACCOUNT_W = PROP_TAG(PT_UNICODE, 14848)
PR_ACCOUNT_A = PROP_TAG(PT_STRING8, 14848)
PR_ALTERNATE_RECIPIENT = PROP_TAG(PT_BINARY, 14849)
PR_CALLBACK_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14850)
PR_CALLBACK_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14850)
PR_CALLBACK_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14850)
PR_CONVERSION_PROHIBITED = PROP_TAG(PT_BOOLEAN, 14851)
PR_DISCLOSE_RECIPIENTS = PROP_TAG(PT_BOOLEAN, 14852)
PR_GENERATION = PROP_TAG(PT_TSTRING, 14853)
PR_GENERATION_W = PROP_TAG(PT_UNICODE, 14853)
PR_GENERATION_A = PROP_TAG(PT_STRING8, 14853)
PR_GIVEN_NAME = PROP_TAG(PT_TSTRING, 14854)
PR_GIVEN_NAME_W = PROP_TAG(PT_UNICODE, 14854)
PR_GIVEN_NAME_A = PROP_TAG(PT_STRING8, 14854)
PR_GOVERNMENT_ID_NUMBER = PROP_TAG(PT_TSTRING, 14855)
PR_GOVERNMENT_ID_NUMBER_W = PROP_TAG(PT_UNICODE, 14855)
PR_GOVERNMENT_ID_NUMBER_A = PROP_TAG(PT_STRING8, 14855)
PR_BUSINESS_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14856)
PR_BUSINESS_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14856)
PR_BUSINESS_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14856)
PR_OFFICE_TELEPHONE_NUMBER = PR_BUSINESS_TELEPHONE_NUMBER
PR_OFFICE_TELEPHONE_NUMBER_W = PR_BUSINESS_TELEPHONE_NUMBER_W
PR_OFFICE_TELEPHONE_NUMBER_A = PR_BUSINESS_TELEPHONE_NUMBER_A
PR_HOME_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14857)
PR_HOME_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14857)
PR_HOME_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14857)
PR_INITIALS = PROP_TAG(PT_TSTRING, 14858)
PR_INITIALS_W = PROP_TAG(PT_UNICODE, 14858)
PR_INITIALS_A = PROP_TAG(PT_STRING8, 14858)
PR_KEYWORD = PROP_TAG(PT_TSTRING, 14859)
PR_KEYWORD_W = PROP_TAG(PT_UNICODE, 14859)
PR_KEYWORD_A = PROP_TAG(PT_STRING8, 14859)
PR_LANGUAGE = PROP_TAG(PT_TSTRING, 14860)
PR_LANGUAGE_W = PROP_TAG(PT_UNICODE, 14860)
PR_LANGUAGE_A = PROP_TAG(PT_STRING8, 14860)
PR_LOCATION = PROP_TAG(PT_TSTRING, 14861)
PR_LOCATION_W = PROP_TAG(PT_UNICODE, 14861)
PR_LOCATION_A = PROP_TAG(PT_STRING8, 14861)
PR_MAIL_PERMISSION = PROP_TAG(PT_BOOLEAN, 14862)
PR_MHS_COMMON_NAME = PROP_TAG(PT_TSTRING, 14863)
PR_MHS_COMMON_NAME_W = PROP_TAG(PT_UNICODE, 14863)
PR_MHS_COMMON_NAME_A = PROP_TAG(PT_STRING8, 14863)
PR_ORGANIZATIONAL_ID_NUMBER = PROP_TAG(PT_TSTRING, 14864)
PR_ORGANIZATIONAL_ID_NUMBER_W = PROP_TAG(PT_UNICODE, 14864)
PR_ORGANIZATIONAL_ID_NUMBER_A = PROP_TAG(PT_STRING8, 14864)
PR_SURNAME = PROP_TAG(PT_TSTRING, 14865)
PR_SURNAME_W = PROP_TAG(PT_UNICODE, 14865)
PR_SURNAME_A = PROP_TAG(PT_STRING8, 14865)
PR_ORIGINAL_ENTRYID = PROP_TAG(PT_BINARY, 14866)
PR_ORIGINAL_DISPLAY_NAME = PROP_TAG(PT_TSTRING, 14867)
PR_ORIGINAL_DISPLAY_NAME_W = PROP_TAG(PT_UNICODE, 14867)
PR_ORIGINAL_DISPLAY_NAME_A = PROP_TAG(PT_STRING8, 14867)
PR_ORIGINAL_SEARCH_KEY = PROP_TAG(PT_BINARY, 14868)
PR_POSTAL_ADDRESS = PROP_TAG(PT_TSTRING, 14869)
PR_POSTAL_ADDRESS_W = PROP_TAG(PT_UNICODE, 14869)
PR_POSTAL_ADDRESS_A = PROP_TAG(PT_STRING8, 14869)
PR_COMPANY_NAME = PROP_TAG(PT_TSTRING, 14870)
PR_COMPANY_NAME_W = PROP_TAG(PT_UNICODE, 14870)
PR_COMPANY_NAME_A = PROP_TAG(PT_STRING8, 14870)
PR_TITLE = PROP_TAG(PT_TSTRING, 14871)
PR_TITLE_W = PROP_TAG(PT_UNICODE, 14871)
PR_TITLE_A = PROP_TAG(PT_STRING8, 14871)
PR_DEPARTMENT_NAME = PROP_TAG(PT_TSTRING, 14872)
PR_DEPARTMENT_NAME_W = PROP_TAG(PT_UNICODE, 14872)
PR_DEPARTMENT_NAME_A = PROP_TAG(PT_STRING8, 14872)
PR_OFFICE_LOCATION = PROP_TAG(PT_TSTRING, 14873)
PR_OFFICE_LOCATION_W = PROP_TAG(PT_UNICODE, 14873)
PR_OFFICE_LOCATION_A = PROP_TAG(PT_STRING8, 14873)
PR_PRIMARY_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14874)
PR_PRIMARY_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14874)
PR_PRIMARY_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14874)
PR_BUSINESS2_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14875)
PR_BUSINESS2_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14875)
PR_BUSINESS2_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14875)
PR_OFFICE2_TELEPHONE_NUMBER = PR_BUSINESS2_TELEPHONE_NUMBER
PR_OFFICE2_TELEPHONE_NUMBER_W = PR_BUSINESS2_TELEPHONE_NUMBER_W
PR_OFFICE2_TELEPHONE_NUMBER_A = PR_BUSINESS2_TELEPHONE_NUMBER_A
PR_MOBILE_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14876)
PR_MOBILE_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14876)
PR_MOBILE_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14876)
PR_CELLULAR_TELEPHONE_NUMBER = PR_MOBILE_TELEPHONE_NUMBER
PR_CELLULAR_TELEPHONE_NUMBER_W = PR_MOBILE_TELEPHONE_NUMBER_W
PR_CELLULAR_TELEPHONE_NUMBER_A = PR_MOBILE_TELEPHONE_NUMBER_A
PR_RADIO_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14877)
PR_RADIO_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14877)
PR_RADIO_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14877)
PR_CAR_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14878)
PR_CAR_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14878)
PR_CAR_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14878)
PR_OTHER_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14879)
PR_OTHER_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14879)
PR_OTHER_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14879)
PR_TRANSMITABLE_DISPLAY_NAME = PROP_TAG(PT_TSTRING, 14880)
PR_TRANSMITABLE_DISPLAY_NAME_W = PROP_TAG(PT_UNICODE, 14880)
PR_TRANSMITABLE_DISPLAY_NAME_A = PROP_TAG(PT_STRING8, 14880)
PR_PAGER_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14881)
PR_PAGER_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14881)
PR_PAGER_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14881)
PR_BEEPER_TELEPHONE_NUMBER = PR_PAGER_TELEPHONE_NUMBER
PR_BEEPER_TELEPHONE_NUMBER_W = PR_PAGER_TELEPHONE_NUMBER_W
PR_BEEPER_TELEPHONE_NUMBER_A = PR_PAGER_TELEPHONE_NUMBER_A
PR_USER_CERTIFICATE = PROP_TAG(PT_BINARY, 14882)
PR_PRIMARY_FAX_NUMBER = PROP_TAG(PT_TSTRING, 14883)
PR_PRIMARY_FAX_NUMBER_W = PROP_TAG(PT_UNICODE, 14883)
PR_PRIMARY_FAX_NUMBER_A = PROP_TAG(PT_STRING8, 14883)
PR_BUSINESS_FAX_NUMBER = PROP_TAG(PT_TSTRING, 14884)
PR_BUSINESS_FAX_NUMBER_W = PROP_TAG(PT_UNICODE, 14884)
PR_BUSINESS_FAX_NUMBER_A = PROP_TAG(PT_STRING8, 14884)
PR_HOME_FAX_NUMBER = PROP_TAG(PT_TSTRING, 14885)
PR_HOME_FAX_NUMBER_W = PROP_TAG(PT_UNICODE, 14885)
PR_HOME_FAX_NUMBER_A = PROP_TAG(PT_STRING8, 14885)
PR_COUNTRY = PROP_TAG(PT_TSTRING, 14886)
PR_COUNTRY_W = PROP_TAG(PT_UNICODE, 14886)
PR_COUNTRY_A = PROP_TAG(PT_STRING8, 14886)
PR_BUSINESS_ADDRESS_COUNTRY = PR_COUNTRY
PR_BUSINESS_ADDRESS_COUNTRY_W = PR_COUNTRY_W
PR_BUSINESS_ADDRESS_COUNTRY_A = PR_COUNTRY_A
PR_LOCALITY = PROP_TAG(PT_TSTRING, 14887)
PR_LOCALITY_W = PROP_TAG(PT_UNICODE, 14887)
PR_LOCALITY_A = PROP_TAG(PT_STRING8, 14887)
PR_BUSINESS_ADDRESS_CITY = PR_LOCALITY
PR_BUSINESS_ADDRESS_CITY_W = PR_LOCALITY_W
PR_BUSINESS_ADDRESS_CITY_A = PR_LOCALITY_A
PR_STATE_OR_PROVINCE = PROP_TAG(PT_TSTRING, 14888)
PR_STATE_OR_PROVINCE_W = PROP_TAG(PT_UNICODE, 14888)
PR_STATE_OR_PROVINCE_A = PROP_TAG(PT_STRING8, 14888)
PR_BUSINESS_ADDRESS_STATE_OR_PROVINCE = PR_STATE_OR_PROVINCE
PR_BUSINESS_ADDRESS_STATE_OR_PROVINCE_W = PR_STATE_OR_PROVINCE_W
PR_BUSINESS_ADDRESS_STATE_OR_PROVINCE_A = PR_STATE_OR_PROVINCE_A
PR_STREET_ADDRESS = PROP_TAG(PT_TSTRING, 14889)
PR_STREET_ADDRESS_W = PROP_TAG(PT_UNICODE, 14889)
PR_STREET_ADDRESS_A = PROP_TAG(PT_STRING8, 14889)
PR_BUSINESS_ADDRESS_STREET = PR_STREET_ADDRESS
PR_BUSINESS_ADDRESS_STREET_W = PR_STREET_ADDRESS_W
PR_BUSINESS_ADDRESS_STREET_A = PR_STREET_ADDRESS_A
PR_POSTAL_CODE = PROP_TAG(PT_TSTRING, 14890)
PR_POSTAL_CODE_W = PROP_TAG(PT_UNICODE, 14890)
PR_POSTAL_CODE_A = PROP_TAG(PT_STRING8, 14890)
PR_BUSINESS_ADDRESS_POSTAL_CODE = PR_POSTAL_CODE
PR_BUSINESS_ADDRESS_POSTAL_CODE_W = PR_POSTAL_CODE_W
PR_BUSINESS_ADDRESS_POSTAL_CODE_A = PR_POSTAL_CODE_A
PR_POST_OFFICE_BOX = PROP_TAG(PT_TSTRING, 14891)
PR_POST_OFFICE_BOX_W = PROP_TAG(PT_UNICODE, 14891)
PR_POST_OFFICE_BOX_A = PROP_TAG(PT_STRING8, 14891)
PR_BUSINESS_ADDRESS_POST_OFFICE_BOX = PR_POST_OFFICE_BOX
PR_BUSINESS_ADDRESS_POST_OFFICE_BOX_W = PR_POST_OFFICE_BOX_W
PR_BUSINESS_ADDRESS_POST_OFFICE_BOX_A = PR_POST_OFFICE_BOX_A
PR_TELEX_NUMBER = PROP_TAG(PT_TSTRING, 14892)
PR_TELEX_NUMBER_W = PROP_TAG(PT_UNICODE, 14892)
PR_TELEX_NUMBER_A = PROP_TAG(PT_STRING8, 14892)
PR_ISDN_NUMBER = PROP_TAG(PT_TSTRING, 14893)
PR_ISDN_NUMBER_W = PROP_TAG(PT_UNICODE, 14893)
PR_ISDN_NUMBER_A = PROP_TAG(PT_STRING8, 14893)
PR_ASSISTANT_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14894)
PR_ASSISTANT_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14894)
PR_ASSISTANT_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14894)
PR_HOME2_TELEPHONE_NUMBER = PROP_TAG(PT_TSTRING, 14895)
PR_HOME2_TELEPHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14895)
PR_HOME2_TELEPHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14895)
PR_ASSISTANT = PROP_TAG(PT_TSTRING, 14896)
PR_ASSISTANT_W = PROP_TAG(PT_UNICODE, 14896)
PR_ASSISTANT_A = PROP_TAG(PT_STRING8, 14896)
PR_SEND_RICH_INFO = PROP_TAG(PT_BOOLEAN, 14912)
PR_WEDDING_ANNIVERSARY = PROP_TAG(PT_SYSTIME, 14913)
PR_BIRTHDAY = PROP_TAG(PT_SYSTIME, 14914)
PR_HOBBIES = PROP_TAG(PT_TSTRING, 14915)
PR_HOBBIES_W = PROP_TAG(PT_UNICODE, 14915)
PR_HOBBIES_A = PROP_TAG(PT_STRING8, 14915)
PR_MIDDLE_NAME = PROP_TAG(PT_TSTRING, 14916)
PR_MIDDLE_NAME_W = PROP_TAG(PT_UNICODE, 14916)
PR_MIDDLE_NAME_A = PROP_TAG(PT_STRING8, 14916)
PR_DISPLAY_NAME_PREFIX = PROP_TAG(PT_TSTRING, 14917)
PR_DISPLAY_NAME_PREFIX_W = PROP_TAG(PT_UNICODE, 14917)
PR_DISPLAY_NAME_PREFIX_A = PROP_TAG(PT_STRING8, 14917)
PR_PROFESSION = PROP_TAG(PT_TSTRING, 14918)
PR_PROFESSION_W = PROP_TAG(PT_UNICODE, 14918)
PR_PROFESSION_A = PROP_TAG(PT_STRING8, 14918)
PR_PREFERRED_BY_NAME = PROP_TAG(PT_TSTRING, 14919)
PR_PREFERRED_BY_NAME_W = PROP_TAG(PT_UNICODE, 14919)
PR_PREFERRED_BY_NAME_A = PROP_TAG(PT_STRING8, 14919)
PR_SPOUSE_NAME = PROP_TAG(PT_TSTRING, 14920)
PR_SPOUSE_NAME_W = PROP_TAG(PT_UNICODE, 14920)
PR_SPOUSE_NAME_A = PROP_TAG(PT_STRING8, 14920)
PR_COMPUTER_NETWORK_NAME = PROP_TAG(PT_TSTRING, 14921)
PR_COMPUTER_NETWORK_NAME_W = PROP_TAG(PT_UNICODE, 14921)
PR_COMPUTER_NETWORK_NAME_A = PROP_TAG(PT_STRING8, 14921)
PR_CUSTOMER_ID = PROP_TAG(PT_TSTRING, 14922)
PR_CUSTOMER_ID_W = PROP_TAG(PT_UNICODE, 14922)
PR_CUSTOMER_ID_A = PROP_TAG(PT_STRING8, 14922)
PR_TTYTDD_PHONE_NUMBER = PROP_TAG(PT_TSTRING, 14923)
PR_TTYTDD_PHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14923)
PR_TTYTDD_PHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14923)
PR_FTP_SITE = PROP_TAG(PT_TSTRING, 14924)
PR_FTP_SITE_W = PROP_TAG(PT_UNICODE, 14924)
PR_FTP_SITE_A = PROP_TAG(PT_STRING8, 14924)
PR_GENDER = PROP_TAG(PT_SHORT, 14925)
PR_MANAGER_NAME = PROP_TAG(PT_TSTRING, 14926)
PR_MANAGER_NAME_W = PROP_TAG(PT_UNICODE, 14926)
PR_MANAGER_NAME_A = PROP_TAG(PT_STRING8, 14926)
PR_NICKNAME = PROP_TAG(PT_TSTRING, 14927)
PR_NICKNAME_W = PROP_TAG(PT_UNICODE, 14927)
PR_NICKNAME_A = PROP_TAG(PT_STRING8, 14927)
PR_PERSONAL_HOME_PAGE = PROP_TAG(PT_TSTRING, 14928)
PR_PERSONAL_HOME_PAGE_W = PROP_TAG(PT_UNICODE, 14928)
PR_PERSONAL_HOME_PAGE_A = PROP_TAG(PT_STRING8, 14928)
PR_BUSINESS_HOME_PAGE = PROP_TAG(PT_TSTRING, 14929)
PR_BUSINESS_HOME_PAGE_W = PROP_TAG(PT_UNICODE, 14929)
PR_BUSINESS_HOME_PAGE_A = PROP_TAG(PT_STRING8, 14929)
PR_CONTACT_VERSION = PROP_TAG(PT_CLSID, 14930)
PR_CONTACT_ENTRYIDS = PROP_TAG(PT_MV_BINARY, 14931)
PR_CONTACT_ADDRTYPES = PROP_TAG(PT_MV_TSTRING, 14932)
PR_CONTACT_ADDRTYPES_W = PROP_TAG(PT_MV_UNICODE, 14932)
PR_CONTACT_ADDRTYPES_A = PROP_TAG(PT_MV_STRING8, 14932)
PR_CONTACT_DEFAULT_ADDRESS_INDEX = PROP_TAG(PT_LONG, 14933)
PR_CONTACT_EMAIL_ADDRESSES = PROP_TAG(PT_MV_TSTRING, 14934)
PR_CONTACT_EMAIL_ADDRESSES_W = PROP_TAG(PT_MV_UNICODE, 14934)
PR_CONTACT_EMAIL_ADDRESSES_A = PROP_TAG(PT_MV_STRING8, 14934)
PR_COMPANY_MAIN_PHONE_NUMBER = PROP_TAG(PT_TSTRING, 14935)
PR_COMPANY_MAIN_PHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 14935)
PR_COMPANY_MAIN_PHONE_NUMBER_A = PROP_TAG(PT_STRING8, 14935)
PR_CHILDRENS_NAMES = PROP_TAG(PT_MV_TSTRING, 14936)
PR_CHILDRENS_NAMES_W = PROP_TAG(PT_MV_UNICODE, 14936)
PR_CHILDRENS_NAMES_A = PROP_TAG(PT_MV_STRING8, 14936)
PR_HOME_ADDRESS_CITY = PROP_TAG(PT_TSTRING, 14937)
PR_HOME_ADDRESS_CITY_W = PROP_TAG(PT_UNICODE, 14937)
PR_HOME_ADDRESS_CITY_A = PROP_TAG(PT_STRING8, 14937)
PR_HOME_ADDRESS_COUNTRY = PROP_TAG(PT_TSTRING, 14938)
PR_HOME_ADDRESS_COUNTRY_W = PROP_TAG(PT_UNICODE, 14938)
PR_HOME_ADDRESS_COUNTRY_A = PROP_TAG(PT_STRING8, 14938)
PR_HOME_ADDRESS_POSTAL_CODE = PROP_TAG(PT_TSTRING, 14939)
PR_HOME_ADDRESS_POSTAL_CODE_W = PROP_TAG(PT_UNICODE, 14939)
PR_HOME_ADDRESS_POSTAL_CODE_A = PROP_TAG(PT_STRING8, 14939)
PR_HOME_ADDRESS_STATE_OR_PROVINCE = PROP_TAG(PT_TSTRING, 14940)
PR_HOME_ADDRESS_STATE_OR_PROVINCE_W = PROP_TAG(PT_UNICODE, 14940)
PR_HOME_ADDRESS_STATE_OR_PROVINCE_A = PROP_TAG(PT_STRING8, 14940)
PR_HOME_ADDRESS_STREET = PROP_TAG(PT_TSTRING, 14941)
PR_HOME_ADDRESS_STREET_W = PROP_TAG(PT_UNICODE, 14941)
PR_HOME_ADDRESS_STREET_A = PROP_TAG(PT_STRING8, 14941)
PR_HOME_ADDRESS_POST_OFFICE_BOX = PROP_TAG(PT_TSTRING, 14942)
PR_HOME_ADDRESS_POST_OFFICE_BOX_W = PROP_TAG(PT_UNICODE, 14942)
PR_HOME_ADDRESS_POST_OFFICE_BOX_A = PROP_TAG(PT_STRING8, 14942)
PR_OTHER_ADDRESS_CITY = PROP_TAG(PT_TSTRING, 14943)
PR_OTHER_ADDRESS_CITY_W = PROP_TAG(PT_UNICODE, 14943)
PR_OTHER_ADDRESS_CITY_A = PROP_TAG(PT_STRING8, 14943)
PR_OTHER_ADDRESS_COUNTRY = PROP_TAG(PT_TSTRING, 14944)
PR_OTHER_ADDRESS_COUNTRY_W = PROP_TAG(PT_UNICODE, 14944)
PR_OTHER_ADDRESS_COUNTRY_A = PROP_TAG(PT_STRING8, 14944)
PR_OTHER_ADDRESS_POSTAL_CODE = PROP_TAG(PT_TSTRING, 14945)
PR_OTHER_ADDRESS_POSTAL_CODE_W = PROP_TAG(PT_UNICODE, 14945)
PR_OTHER_ADDRESS_POSTAL_CODE_A = PROP_TAG(PT_STRING8, 14945)
PR_OTHER_ADDRESS_STATE_OR_PROVINCE = PROP_TAG(PT_TSTRING, 14946)
PR_OTHER_ADDRESS_STATE_OR_PROVINCE_W = PROP_TAG(PT_UNICODE, 14946)
PR_OTHER_ADDRESS_STATE_OR_PROVINCE_A = PROP_TAG(PT_STRING8, 14946)
PR_OTHER_ADDRESS_STREET = PROP_TAG(PT_TSTRING, 14947)
PR_OTHER_ADDRESS_STREET_W = PROP_TAG(PT_UNICODE, 14947)
PR_OTHER_ADDRESS_STREET_A = PROP_TAG(PT_STRING8, 14947)
PR_OTHER_ADDRESS_POST_OFFICE_BOX = PROP_TAG(PT_TSTRING, 14948)
PR_OTHER_ADDRESS_POST_OFFICE_BOX_W = PROP_TAG(PT_UNICODE, 14948)
PR_OTHER_ADDRESS_POST_OFFICE_BOX_A = PROP_TAG(PT_STRING8, 14948)
PR_STORE_PROVIDERS = PROP_TAG(PT_BINARY, 15616)
PR_AB_PROVIDERS = PROP_TAG(PT_BINARY, 15617)
PR_TRANSPORT_PROVIDERS = PROP_TAG(PT_BINARY, 15618)
PR_DEFAULT_PROFILE = PROP_TAG(PT_BOOLEAN, 15620)
PR_AB_SEARCH_PATH = PROP_TAG(PT_MV_BINARY, 15621)
PR_AB_DEFAULT_DIR = PROP_TAG(PT_BINARY, 15622)
PR_AB_DEFAULT_PAB = PROP_TAG(PT_BINARY, 15623)
PR_FILTERING_HOOKS = PROP_TAG(PT_BINARY, 15624)
PR_SERVICE_NAME = PROP_TAG(PT_TSTRING, 15625)
PR_SERVICE_NAME_W = PROP_TAG(PT_UNICODE, 15625)
PR_SERVICE_NAME_A = PROP_TAG(PT_STRING8, 15625)
PR_SERVICE_DLL_NAME = PROP_TAG(PT_TSTRING, 15626)
PR_SERVICE_DLL_NAME_W = PROP_TAG(PT_UNICODE, 15626)
PR_SERVICE_DLL_NAME_A = PROP_TAG(PT_STRING8, 15626)
PR_SERVICE_ENTRY_NAME = PROP_TAG(PT_STRING8, 15627)
PR_SERVICE_UID = PROP_TAG(PT_BINARY, 15628)
PR_SERVICE_EXTRA_UIDS = PROP_TAG(PT_BINARY, 15629)
PR_SERVICES = PROP_TAG(PT_BINARY, 15630)
PR_SERVICE_SUPPORT_FILES = PROP_TAG(PT_MV_TSTRING, 15631)
PR_SERVICE_SUPPORT_FILES_W = PROP_TAG(PT_MV_UNICODE, 15631)
PR_SERVICE_SUPPORT_FILES_A = PROP_TAG(PT_MV_STRING8, 15631)
PR_SERVICE_DELETE_FILES = PROP_TAG(PT_MV_TSTRING, 15632)
PR_SERVICE_DELETE_FILES_W = PROP_TAG(PT_MV_UNICODE, 15632)
PR_SERVICE_DELETE_FILES_A = PROP_TAG(PT_MV_STRING8, 15632)
PR_AB_SEARCH_PATH_UPDATE = PROP_TAG(PT_BINARY, 15633)
PR_PROFILE_NAME = PROP_TAG(PT_TSTRING, 15634)
PR_PROFILE_NAME_A = PROP_TAG(PT_STRING8, 15634)
PR_PROFILE_NAME_W = PROP_TAG(PT_UNICODE, 15634)
PR_IDENTITY_DISPLAY = PROP_TAG(PT_TSTRING, 15872)
PR_IDENTITY_DISPLAY_W = PROP_TAG(PT_UNICODE, 15872)
PR_IDENTITY_DISPLAY_A = PROP_TAG(PT_STRING8, 15872)
PR_IDENTITY_ENTRYID = PROP_TAG(PT_BINARY, 15873)
PR_RESOURCE_METHODS = PROP_TAG(PT_LONG, 15874)
PR_RESOURCE_TYPE = PROP_TAG(PT_LONG, 15875)
PR_STATUS_CODE = PROP_TAG(PT_LONG, 15876)
PR_IDENTITY_SEARCH_KEY = PROP_TAG(PT_BINARY, 15877)
PR_OWN_STORE_ENTRYID = PROP_TAG(PT_BINARY, 15878)
PR_RESOURCE_PATH = PROP_TAG(PT_TSTRING, 15879)
PR_RESOURCE_PATH_W = PROP_TAG(PT_UNICODE, 15879)
PR_RESOURCE_PATH_A = PROP_TAG(PT_STRING8, 15879)
PR_STATUS_STRING = PROP_TAG(PT_TSTRING, 15880)
PR_STATUS_STRING_W = PROP_TAG(PT_UNICODE, 15880)
PR_STATUS_STRING_A = PROP_TAG(PT_STRING8, 15880)
PR_X400_DEFERRED_DELIVERY_CANCEL = PROP_TAG(PT_BOOLEAN, 15881)
PR_HEADER_FOLDER_ENTRYID = PROP_TAG(PT_BINARY, 15882)
PR_REMOTE_PROGRESS = PROP_TAG(PT_LONG, 15883)
PR_REMOTE_PROGRESS_TEXT = PROP_TAG(PT_TSTRING, 15884)
PR_REMOTE_PROGRESS_TEXT_W = PROP_TAG(PT_UNICODE, 15884)
PR_REMOTE_PROGRESS_TEXT_A = PROP_TAG(PT_STRING8, 15884)
PR_REMOTE_VALIDATE_OK = PROP_TAG(PT_BOOLEAN, 15885)
PR_CONTROL_FLAGS = PROP_TAG(PT_LONG, 16128)
PR_CONTROL_STRUCTURE = PROP_TAG(PT_BINARY, 16129)
PR_CONTROL_TYPE = PROP_TAG(PT_LONG, 16130)
PR_DELTAX = PROP_TAG(PT_LONG, 16131)
PR_DELTAY = PROP_TAG(PT_LONG, 16132)
PR_XPOS = PROP_TAG(PT_LONG, 16133)
PR_YPOS = PROP_TAG(PT_LONG, 16134)
PR_CONTROL_ID = PROP_TAG(PT_BINARY, 16135)
PR_INITIAL_DETAILS_PANE = PROP_TAG(PT_LONG, 16136)

PROP_ID_SECURE_MIN = 26608
PROP_ID_SECURE_MAX = 26623

# From EDKMDB.H
pidExchangeXmitReservedMin = 16352
pidExchangeNonXmitReservedMin = 26080
pidProfileMin = 26112
pidStoreMin = 26136
pidFolderMin = 26168
pidMessageReadOnlyMin = 26176
pidMessageWriteableMin = 26200
pidAttachReadOnlyMin = 26220
pidSpecialMin = 26224
pidAdminMin = 26256
pidSecureProfileMin = PROP_ID_SECURE_MIN

PR_PROFILE_VERSION = PROP_TAG(PT_LONG, pidProfileMin + 0)
PR_PROFILE_CONFIG_FLAGS = PROP_TAG(PT_LONG, pidProfileMin + 1)
PR_PROFILE_HOME_SERVER = PROP_TAG(PT_STRING8, pidProfileMin + 2)
PR_PROFILE_HOME_SERVER_DN = PROP_TAG(PT_STRING8, pidProfileMin + 18)
PR_PROFILE_HOME_SERVER_ADDRS = PROP_TAG(PT_MV_STRING8, pidProfileMin + 19)
PR_PROFILE_USER = PROP_TAG(PT_STRING8, pidProfileMin + 3)
PR_PROFILE_CONNECT_FLAGS = PROP_TAG(PT_LONG, pidProfileMin + 4)
PR_PROFILE_TRANSPORT_FLAGS = PROP_TAG(PT_LONG, pidProfileMin + 5)
PR_PROFILE_UI_STATE = PROP_TAG(PT_LONG, pidProfileMin + 6)
PR_PROFILE_UNRESOLVED_NAME = PROP_TAG(PT_STRING8, pidProfileMin + 7)
PR_PROFILE_UNRESOLVED_SERVER = PROP_TAG(PT_STRING8, pidProfileMin + 8)
PR_PROFILE_BINDING_ORDER = PROP_TAG(PT_STRING8, pidProfileMin + 9)
PR_PROFILE_MAX_RESTRICT = PROP_TAG(PT_LONG, pidProfileMin + 13)
PR_PROFILE_AB_FILES_PATH = PROP_TAG(PT_STRING8, pidProfileMin + 14)
PR_PROFILE_OFFLINE_STORE_PATH = PROP_TAG(PT_STRING8, pidProfileMin + 16)
PR_PROFILE_OFFLINE_INFO = PROP_TAG(PT_BINARY, pidProfileMin + 17)
PR_PROFILE_ADDR_INFO = PROP_TAG(PT_BINARY, pidSpecialMin + 23)
PR_PROFILE_OPTIONS_DATA = PROP_TAG(PT_BINARY, pidSpecialMin + 25)
PR_PROFILE_SECURE_MAILBOX = PROP_TAG(PT_BINARY, pidSecureProfileMin + 0)
PR_DISABLE_WINSOCK = PROP_TAG(PT_LONG, pidProfileMin + 24)
PR_OST_ENCRYPTION = PROP_TAG(PT_LONG, 26370)
PR_PROFILE_OPEN_FLAGS = PROP_TAG(PT_LONG, pidProfileMin + 9)
PR_PROFILE_TYPE = PROP_TAG(PT_LONG, pidProfileMin + 10)
PR_PROFILE_MAILBOX = PROP_TAG(PT_STRING8, pidProfileMin + 11)
PR_PROFILE_SERVER = PROP_TAG(PT_STRING8, pidProfileMin + 12)
PR_PROFILE_SERVER_DN = PROP_TAG(PT_STRING8, pidProfileMin + 20)
PR_PROFILE_FAVFLD_DISPLAY_NAME = PROP_TAG(PT_STRING8, pidProfileMin + 15)
PR_PROFILE_FAVFLD_COMMENT = PROP_TAG(PT_STRING8, pidProfileMin + 21)
PR_PROFILE_ALLPUB_DISPLAY_NAME = PROP_TAG(PT_STRING8, pidProfileMin + 22)
PR_PROFILE_ALLPUB_COMMENT = PROP_TAG(PT_STRING8, pidProfileMin + 23)

OSTF_NO_ENCRYPTION = -2147483648
OSTF_COMPRESSABLE_ENCRYPTION = 1073741824
OSTF_BEST_ENCRYPTION = 536870912


PR_NON_IPM_SUBTREE_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 8)
PR_EFORMS_REGISTRY_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 9)
PR_SPLUS_FREE_BUSY_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 10)
PR_OFFLINE_ADDRBOOK_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 11)
PR_EFORMS_FOR_LOCALE_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 12)
PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 13)
PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 14)
PR_OFFLINE_MESSAGE_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 15)
PR_IPM_FAVORITES_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 24)
PR_IPM_PUBLIC_FOLDERS_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 25)
PR_GW_MTSIN_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 16)
PR_GW_MTSOUT_ENTRYID = PROP_TAG(PT_BINARY, pidStoreMin + 17)
PR_TRANSFER_ENABLED = PROP_TAG(PT_BOOLEAN, pidStoreMin + 18)
PR_TEST_LINE_SPEED = PROP_TAG(PT_BINARY, pidStoreMin + 19)
PR_HIERARCHY_SYNCHRONIZER = PROP_TAG(PT_OBJECT, pidStoreMin + 20)
PR_CONTENTS_SYNCHRONIZER = PROP_TAG(PT_OBJECT, pidStoreMin + 21)
PR_COLLECTOR = PROP_TAG(PT_OBJECT, pidStoreMin + 22)
PR_FAST_TRANSFER = PROP_TAG(PT_OBJECT, pidStoreMin + 23)
PR_STORE_OFFLINE = PROP_TAG(PT_BOOLEAN, pidStoreMin + 26)
PR_IN_TRANSIT = PROP_TAG(PT_BOOLEAN, pidStoreMin)
PR_REPLICATION_STYLE = PROP_TAG(PT_LONG, pidAdminMin)
PR_REPLICATION_SCHEDULE = PROP_TAG(PT_BINARY, pidAdminMin + 1)
PR_REPLICATION_MESSAGE_PRIORITY = PROP_TAG(PT_LONG, pidAdminMin + 2)
PR_OVERALL_MSG_AGE_LIMIT = PROP_TAG(PT_LONG, pidAdminMin + 3)
PR_REPLICATION_ALWAYS_INTERVAL = PROP_TAG(PT_LONG, pidAdminMin + 4)
PR_REPLICATION_MSG_SIZE = PROP_TAG(PT_LONG, pidAdminMin + 5)
STYLE_ALWAYS_INTERVAL_DEFAULT = 15
REPLICATION_MESSAGE_SIZE_LIMIT_DEFAULT = 100
STYLE_NEVER = 0
STYLE_NORMAL = 1
STYLE_ALWAYS = 2
STYLE_DEFAULT = -1
PR_SOURCE_KEY = PROP_TAG(PT_BINARY, pidExchangeNonXmitReservedMin + 0)
PR_PARENT_SOURCE_KEY = PROP_TAG(PT_BINARY, pidExchangeNonXmitReservedMin + 1)
PR_CHANGE_KEY = PROP_TAG(PT_BINARY, pidExchangeNonXmitReservedMin + 2)
PR_PREDECESSOR_CHANGE_LIST = PROP_TAG(PT_BINARY, pidExchangeNonXmitReservedMin + 3)
PR_FOLDER_CHILD_COUNT = PROP_TAG(PT_LONG, pidFolderMin)
PR_RIGHTS = PROP_TAG(PT_LONG, pidFolderMin + 1)
PR_ACL_TABLE = PROP_TAG(PT_OBJECT, pidExchangeXmitReservedMin)
PR_RULES_TABLE = PROP_TAG(PT_OBJECT, pidExchangeXmitReservedMin + 1)
PR_HAS_RULES = PROP_TAG(PT_BOOLEAN, pidFolderMin + 2)
PR_ADDRESS_BOOK_ENTRYID = PROP_TAG(PT_BINARY, pidFolderMin + 3)
PR_ACL_DATA = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin)
PR_RULES_DATA = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin + 1)
PR_FOLDER_DESIGN_FLAGS = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 2)
PR_DESIGN_IN_PROGRESS = PROP_TAG(PT_BOOLEAN, pidExchangeXmitReservedMin + 4)
PR_SECURE_ORIGINATION = PROP_TAG(PT_BOOLEAN, pidExchangeXmitReservedMin + 5)
PR_PUBLISH_IN_ADDRESS_BOOK = PROP_TAG(PT_BOOLEAN, pidExchangeXmitReservedMin + 6)
PR_RESOLVE_METHOD = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 7)
PR_ADDRESS_BOOK_DISPLAY_NAME = PROP_TAG(PT_TSTRING, pidExchangeXmitReservedMin + 8)
PR_EFORMS_LOCALE_ID = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 9)
PR_REPLICA_LIST = PROP_TAG(PT_BINARY, pidAdminMin + 8)
PR_OVERALL_AGE_LIMIT = PROP_TAG(PT_LONG, pidAdminMin + 9)
RESOLVE_METHOD_DEFAULT = 0
RESOLVE_METHOD_LAST_WRITER_WINS = 1
RESOLVE_METHOD_NO_CONFLICT_NOTIFICATION = 2
PR_PUBLIC_FOLDER_ENTRYID = PROP_TAG(PT_BINARY, pidFolderMin + 4)
PR_HAS_NAMED_PROPERTIES = PROP_TAG(PT_BOOLEAN, pidMessageReadOnlyMin + 10)
PR_CREATOR_NAME = PROP_TAG(PT_TSTRING, pidExchangeXmitReservedMin + 24)
PR_CREATOR_ENTRYID = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin + 25)
PR_LAST_MODIFIER_NAME = PROP_TAG(PT_TSTRING, pidExchangeXmitReservedMin + 26)
PR_LAST_MODIFIER_ENTRYID = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin + 27)
PR_HAS_DAMS = PROP_TAG(PT_BOOLEAN, pidExchangeXmitReservedMin + 10)
PR_RULE_TRIGGER_HISTORY = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin + 18)
PR_MOVE_TO_STORE_ENTRYID = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin + 19)
PR_MOVE_TO_FOLDER_ENTRYID = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin + 20)
PR_REPLICA_SERVER = PROP_TAG(PT_TSTRING, pidMessageReadOnlyMin + 4)
PR_DEFERRED_SEND_NUMBER = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 11)
PR_DEFERRED_SEND_UNITS = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 12)
PR_EXPIRY_NUMBER = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 13)
PR_EXPIRY_UNITS = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 14)
PR_DEFERRED_SEND_TIME = PROP_TAG(PT_SYSTIME, pidExchangeXmitReservedMin + 15)
PR_GW_ADMIN_OPERATIONS = PROP_TAG(PT_LONG, pidMessageWriteableMin)
PR_P1_CONTENT = PROP_TAG(PT_BINARY, 4352)
PR_P1_CONTENT_TYPE = PROP_TAG(PT_BINARY, 4353)
PR_CLIENT_ACTIONS = PROP_TAG(PT_BINARY, pidMessageReadOnlyMin + 5)
PR_DAM_ORIGINAL_ENTRYID = PROP_TAG(PT_BINARY, pidMessageReadOnlyMin + 6)
PR_DAM_BACK_PATCHED = PROP_TAG(PT_BOOLEAN, pidMessageReadOnlyMin + 7)
PR_RULE_ERROR = PROP_TAG(PT_LONG, pidMessageReadOnlyMin + 8)
PR_RULE_ACTION_TYPE = PROP_TAG(PT_LONG, pidMessageReadOnlyMin + 9)
PR_RULE_ACTION_NUMBER = PROP_TAG(PT_LONG, pidMessageReadOnlyMin + 16)
PR_RULE_FOLDER_ENTRYID = PROP_TAG(PT_BINARY, pidMessageReadOnlyMin + 17)
PR_CONFLICT_ENTRYID = PROP_TAG(PT_BINARY, pidExchangeXmitReservedMin + 16)
PR_MESSAGE_LOCALE_ID = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 17)
PR_STORAGE_QUOTA_LIMIT = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 21)
PR_EXCESS_STORAGE_USED = PROP_TAG(PT_LONG, pidExchangeXmitReservedMin + 22)
PR_SVR_GENERATING_QUOTA_MSG = PROP_TAG(PT_TSTRING, pidExchangeXmitReservedMin + 23)
PR_DELEGATED_BY_RULE = PROP_TAG(PT_BOOLEAN, pidExchangeXmitReservedMin + 3)
MSGSTATUS_IN_CONFLICT = 2048
PR_IN_CONFLICT = PROP_TAG(PT_BOOLEAN, pidAttachReadOnlyMin)
PR_LONGTERM_ENTRYID_FROM_TABLE = PROP_TAG(PT_BINARY, pidSpecialMin)
PR_ORIGINATOR_NAME = PROP_TAG(PT_TSTRING, pidMessageWriteableMin + 3)
PR_ORIGINATOR_ADDR = PROP_TAG(PT_TSTRING, pidMessageWriteableMin + 4)
PR_ORIGINATOR_ADDRTYPE = PROP_TAG(PT_TSTRING, pidMessageWriteableMin + 5)
PR_ORIGINATOR_ENTRYID = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 6)
PR_ARRIVAL_TIME = PROP_TAG(PT_SYSTIME, pidMessageWriteableMin + 7)
PR_TRACE_INFO = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 8)
PR_INTERNAL_TRACE_INFO = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 18)
PR_SUBJECT_TRACE_INFO = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 9)
PR_RECIPIENT_NUMBER = PROP_TAG(PT_LONG, pidMessageWriteableMin + 10)
PR_MTS_SUBJECT_ID = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 11)
PR_REPORT_DESTINATION_NAME = PROP_TAG(PT_TSTRING, pidMessageWriteableMin + 12)
PR_REPORT_DESTINATION_ENTRYID = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 13)
PR_CONTENT_SEARCH_KEY = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 14)
PR_FOREIGN_ID = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 15)
PR_FOREIGN_REPORT_ID = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 16)
PR_FOREIGN_SUBJECT_ID = PROP_TAG(PT_BINARY, pidMessageWriteableMin + 17)
PR_MTS_ID = PR_MESSAGE_SUBMISSION_ID
PR_MTS_REPORT_ID = PR_MESSAGE_SUBMISSION_ID

PR_FOLDER_FLAGS = PROP_TAG(PT_LONG, pidAdminMin + 24)
PR_LAST_ACCESS_TIME = PROP_TAG(PT_SYSTIME, pidAdminMin + 25)
PR_RESTRICTION_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 26)
PR_CATEG_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 27)
PR_CACHED_COLUMN_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 28)
PR_NORMAL_MSG_W_ATTACH_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 29)
PR_ASSOC_MSG_W_ATTACH_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 30)
PR_RECIPIENT_ON_NORMAL_MSG_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 31)
PR_RECIPIENT_ON_ASSOC_MSG_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 32)
PR_ATTACH_ON_NORMAL_MSG_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 33)
PR_ATTACH_ON_ASSOC_MSG_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 34)
PR_NORMAL_MESSAGE_SIZE = PROP_TAG(PT_LONG, pidAdminMin + 35)
PR_NORMAL_MESSAGE_SIZE_EXTENDED = PROP_TAG(PT_I8, pidAdminMin + 35)
PR_ASSOC_MESSAGE_SIZE = PROP_TAG(PT_LONG, pidAdminMin + 36)
PR_ASSOC_MESSAGE_SIZE_EXTENDED = PROP_TAG(PT_I8, pidAdminMin + 36)
PR_FOLDER_PATHNAME = PROP_TAG(PT_TSTRING, pidAdminMin + 37)
PR_OWNER_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 38)
PR_CONTACT_COUNT = PROP_TAG(PT_LONG, pidAdminMin + 39)

PR_MESSAGE_SIZE_EXTENDED = PROP_TAG(PT_I8, PROP_ID(PR_MESSAGE_SIZE))

PR_USERFIELDS = PROP_TAG(PT_BINARY, 0x36E3)

# IExchangeManageStoreEx::CreateStoreEntryID2
PR_FORCE_USE_ENTRYID_SERVER = PROP_TAG(PT_BOOLEAN, 0x7CFE)
PR_PROFILE_MDB_DN = PROP_TAG(PT_STRING8, 0x7CFF)

# MSPST.h
PST_EXTERN_PROPID_BASE = 0x6700
PR_PST_PATH = PROP_TAG(PT_STRING8, PST_EXTERN_PROPID_BASE + 0)
PR_PST_PATH_W = PROP_TAG(PT_UNICODE, PST_EXTERN_PROPID_BASE + 0)
PR_PST_PATH_A = PROP_TAG(PT_STRING8, PST_EXTERN_PROPID_BASE + 0)
PR_PST_REMEMBER_PW = PROP_TAG(PT_BOOLEAN, PST_EXTERN_PROPID_BASE + 1)
PR_PST_ENCRYPTION = PROP_TAG(PT_LONG, PST_EXTERN_PROPID_BASE + 2)
PR_PST_PW_SZ_OLD = PROP_TAG(PT_STRING8, PST_EXTERN_PROPID_BASE + 3)
PR_PST_PW_SZ_OLD_W = PROP_TAG(PT_UNICODE, PST_EXTERN_PROPID_BASE + 3)
PR_PST_PW_SZ_OLD_A = PROP_TAG(PT_STRING8, PST_EXTERN_PROPID_BASE + 3)
PR_PST_PW_SZ_NEW = PROP_TAG(PT_STRING8, PST_EXTERN_PROPID_BASE + 4)
PR_PST_PW_SZ_NEW_W = PROP_TAG(PT_UNICODE, PST_EXTERN_PROPID_BASE + 4)
PR_PST_PW_SZ_NEW_A = PROP_TAG(PT_STRING8, PST_EXTERN_PROPID_BASE + 4)



# Test dynamic policy, and running object table.

import pythoncom
import winerror

from win32com.server.exception import Exception

error = "testDynamic error"

iid = pythoncom.MakeIID("{b48969a0-784b-11d0-ae71-d23f56000000}")


class VeryPermissive:
    def _dynamic_(self, name, lcid, wFlags, args):
        if wFlags & pythoncom.DISPATCH_METHOD:
            return getattr(self, name)(*args)

        if wFlags & pythoncom.DISPATCH_PROPERTYGET:
            try:
                # to avoid problems with byref param handling, tuple results are converted to lists.
                ret = self.__dict__[name]
                if type(ret) == type(()):
                    ret = list(ret)
                return ret
            except KeyError:  # Probably a method request.
                raise Exception(scode=winerror.DISP_E_MEMBERNOTFOUND)

        if wFlags & (
            pythoncom.DISPATCH_PROPERTYPUT | pythoncom.DISPATCH_PROPERTYPUTREF
        ):
            setattr(self, name, args[0])
            return

        raise Exception(scode=winerror.E_INVALIDARG, desc="invalid wFlags")

    def write(self, *args):
        if len(args) == 0:
            raise Exception(
                scode=winerror.DISP_E_BADPARAMCOUNT
            )  # Probably call as PROPGET.

        for arg in args[:-1]:
            print(str(arg), end=" ")
        print(str(args[-1]))


def Test():
    import win32com.server.util, win32com.server.policy

    #       import win32dbg;win32dbg.brk()
    ob = win32com.server.util.wrap(
        VeryPermissive(), usePolicy=win32com.server.policy.DynamicPolicy
    )
    try:
        handle = pythoncom.RegisterActiveObject(ob, iid, 0)
    except pythoncom.com_error as details:
        print("Warning - could not register the object in the ROT:", details)
        handle = None
    try:
        import win32com.client.dynamic

        client = win32com.client.dynamic.Dispatch(iid)
        client.ANewAttr = "Hello"
        if client.ANewAttr != "Hello":
            raise error("Could not set dynamic property")

        v = ["Hello", "From", "Python", 1.4]
        client.TestSequence = v
        if v != list(client.TestSequence):
            raise error(
                "Dynamic sequences not working! %r/%r"
                % (repr(v), repr(client.testSequence))
            )

        client.write("This", "output", "has", "come", "via", "testDynamic.py")
        # Check our new "_FlagAsMethod" works (kinda!)
        client._FlagAsMethod("NotReallyAMethod")
        if not callable(client.NotReallyAMethod):
            raise error("Method I flagged as callable isn't!")

        client = None
    finally:
        if handle is not None:
            pythoncom.RevokeActiveObject(handle)
    print("Test worked!")


if __name__ == "__main__":
    Test()



import sys
from win32com.axscript.server.error import Exception
from win32com.axscript import axscript
from win32com.axscript.server import axsite
import pythoncom
from win32com.server import util, connect
import win32com.server.policy


class MySite(axsite.AXSite):
    def OnScriptError(self, error):
        exc = error.GetExceptionInfo()
        context, line, char = error.GetSourcePosition()
        print(" >Exception:", exc[1])
        try:
            st = error.GetSourceLineText()
        except pythoncom.com_error:
            st = None
        if st is None:
            st = ""
        text = st + "\n" + (" " * (char - 1)) + "^" + "\n" + exc[2]
        for line in text.splitlines():
            print("  >" + line)


class MyCollection(util.Collection):
    def _NewEnum(self):
        print("Making new Enumerator")
        return util.Collection._NewEnum(self)


class Test:
    _public_methods_ = ["echo"]
    _public_attrs_ = ["collection", "verbose"]

    def __init__(self):
        self.verbose = 0
        self.collection = util.wrap(MyCollection([1, "Two", 3]))
        self.last = ""

    #    self._connect_server_ = TestConnectServer(self)

    def echo(self, *args):
        self.last = "".join(map(str, args))
        if self.verbose:
            for arg in args:
                print(arg, end=" ")
            print()


#    self._connect_server_.Broadcast(last)


#### Connections currently wont work, as there is no way for the engine to
#### know what events we support.  We need typeinfo support.

IID_ITestEvents = pythoncom.MakeIID("{8EB72F90-0D44-11d1-9C4B-00AA00125A98}")


class TestConnectServer(connect.ConnectableServer):
    _connect_interfaces_ = [IID_ITestEvents]
    # The single public method that the client can call on us
    # (ie, as a normal COM server, this exposes just this single method.
    def __init__(self, object):
        self.object = object

    def Broadcast(self, arg):
        # Simply broadcast a notification.
        self._BroadcastNotify(self.NotifyDoneIt, (arg,))

    def NotifyDoneIt(self, interface, arg):
        interface.Invoke(1000, 0, pythoncom.DISPATCH_METHOD, 1, arg)


VBScript = """\
prop = "Property Value"

sub hello(arg1)
   test.echo arg1
end sub
  
sub testcollection
   test.verbose = 1
   for each item in test.collection
     test.echo "Collection item is", item
   next
end sub
"""
if sys.version_info < (3,):
    PyScript = """print "PyScript is being parsed..."\n"""
else:
    PyScript = """print("PyScript is being parsed...")\n"""
PyScript += """\
prop = "Property Value"
def hello(arg1):
   test.echo(arg1)
   pass
   
def testcollection():
   test.verbose = 1
#   test.collection[1] = "New one"
   for item in test.collection:
     test.echo("Collection item is", item)
   pass
"""

ErrScript = """\
bad code for everyone!
"""


def TestEngine(engineName, code, bShouldWork=1):
    echoer = Test()
    model = {
        "test": util.wrap(echoer),
    }

    site = MySite(model)
    engine = site._AddEngine(engineName)
    engine.AddCode(code, axscript.SCRIPTTEXT_ISPERSISTENT)
    try:
        engine.Start()
    finally:
        if not bShouldWork:
            engine.Close()
            return
    doTestEngine(engine, echoer)
    # re-transition the engine back to the UNINITIALIZED state, a-la ASP.
    engine.eScript.SetScriptState(axscript.SCRIPTSTATE_UNINITIALIZED)
    engine.eScript.SetScriptSite(util.wrap(site))
    print("restarting")
    engine.Start()
    # all done!
    engine.Close()


def doTestEngine(engine, echoer):
    # Now call into the scripts IDispatch
    from win32com.client.dynamic import Dispatch

    ob = Dispatch(engine.GetScriptDispatch())
    try:
        ob.hello("Goober")
    except pythoncom.com_error as exc:
        print("***** Calling 'hello' failed", exc)
        return
    if echoer.last != "Goober":
        print("***** Function call didnt set value correctly", repr(echoer.last))

    if str(ob.prop) != "Property Value":
        print("***** Property Value not correct - ", repr(ob.prop))

    ob.testcollection()

    # Now make sure my engines can evaluate stuff.
    result = engine.eParse.ParseScriptText(
        "1+1", None, None, None, 0, 0, axscript.SCRIPTTEXT_ISEXPRESSION
    )
    if result != 2:
        print("Engine could not evaluate '1+1' - said the result was", result)


def dotestall():
    for i in range(10):
        TestEngine("Python", PyScript)
        print(sys.gettotalrefcount())


##  print "Testing Exceptions"
##  try:
##    TestEngine("Python", ErrScript, 0)
##  except pythoncom.com_error:
##    pass


def testall():
    dotestall()
    pythoncom.CoUninitialize()
    print(
        "AXScript Host worked correctly - %d/%d COM objects left alive."
        % (pythoncom._GetInterfaceCount(), pythoncom._GetGatewayCount())
    )


if __name__ == "__main__":
    testall()



# A sample originally provided by Richard Bell, and modified by Mark Hammond.

# This sample demonstrates how to use COM events in an aparment-threaded
# world.  In this world, COM itself ensures that all calls to and events
# from an object happen on the same thread that created the object, even
# if they originated from different threads.  For this cross-thread
# marshalling to work, this main thread *must* run a "message-loop" (ie,
# a loop fetching and dispatching Windows messages).  Without such message
# processing, dead-locks can occur.

# See also eventsFreeThreaded.py for how to do this in a free-threaded
# world where these marshalling considerations do not exist.

# NOTE: This example uses Internet Explorer, but it should not be considerd
# a "best-practices" for writing against IE events, but for working with
# events in general. For example:
# * The first OnDocumentComplete event is not a reliable indicator that the
#   URL has completed loading
# * As we are demonstrating the most efficient way of handling events, when
#   running this sample you will see an IE Windows briefly appear, but
#   vanish without ever being repainted.

import sys
import os
import win32com.client
import win32api
import win32event

# sys.coinit_flags not set, so pythoncom initializes apartment-threaded.
import pythoncom
import time


class ExplorerEvents:
    def __init__(self):
        self.event = win32event.CreateEvent(None, 0, 0, None)

    def OnDocumentComplete(self, pDisp=pythoncom.Empty, URL=pythoncom.Empty):
        thread = win32api.GetCurrentThreadId()
        print("OnDocumentComplete event processed on thread %d" % thread)
        # Set the event our main thread is waiting on.
        win32event.SetEvent(self.event)

    def OnQuit(self):
        thread = win32api.GetCurrentThreadId()
        print("OnQuit event processed on thread %d" % thread)
        win32event.SetEvent(self.event)


def WaitWhileProcessingMessages(event, timeout=2):
    start = time.clock()
    while True:
        # Wake 4 times a second - we can't just specify the
        # full timeout here, as then it would reset for every
        # message we process.
        rc = win32event.MsgWaitForMultipleObjects(
            (event,), 0, 250, win32event.QS_ALLEVENTS
        )
        if rc == win32event.WAIT_OBJECT_0:
            # event signalled - stop now!
            return True
        if (time.clock() - start) > timeout:
            # Timeout expired.
            return False
        # must be a message.
        pythoncom.PumpWaitingMessages()


def TestExplorerEvents():
    iexplore = win32com.client.DispatchWithEvents(
        "InternetExplorer.Application", ExplorerEvents
    )

    thread = win32api.GetCurrentThreadId()
    print("TestExplorerEvents created IE object on thread %d" % thread)

    iexplore.Visible = 1
    try:
        iexplore.Navigate(win32api.GetFullPathName("..\\readme.html"))
    except pythoncom.com_error as details:
        print("Warning - could not open the test HTML file", details)

    # Wait for the event to be signalled while pumping messages.
    if not WaitWhileProcessingMessages(iexplore.event):
        print("Document load event FAILED to fire!!!")

    iexplore.Quit()
    #
    # Give IE a chance to shutdown, else it can get upset on fast machines.
    # Note, Quit generates events.  Although this test does NOT catch them
    # it is NECESSARY to pump messages here instead of a sleep so that the Quit
    # happens properly!
    if not WaitWhileProcessingMessages(iexplore.event):
        print("OnQuit event FAILED to fire!!!")

    iexplore = None


if __name__ == "__main__":
    TestExplorerEvents()



import win32security, win32event

evt = win32event.CreateEvent(None, 0, 0, None)
win32security.LsaRegisterPolicyChangeNotification(
    win32security.PolicyNotifyAuditEventsInformation, evt
)
print("Waiting for you change Audit policy in Management console ...")
ret_code = win32event.WaitForSingleObject(evt, 1000000000)
## should come back when you change Audit policy in Management console ...
print(ret_code)
win32security.LsaUnregisterPolicyChangeNotification(
    win32security.PolicyNotifyAuditEventsInformation, evt
)



# ocxserialtest.py
#
# Sample that uses the mscomm OCX to talk to a serial
# device.

# Very simple -  queries a modem for ATI responses

import win32ui, win32uiole
import win32con
from pywin.mfc import dialog, activex
from win32com.client import gencache
import pythoncom

SERIAL_SETTINGS = "19200,n,8,1"
SERIAL_PORT = 2

win32ui.DoWaitCursor(1)
serialModule = gencache.EnsureModule("{648A5603-2C6E-101B-82B6-000000000014}", 0, 1, 1)
win32ui.DoWaitCursor(0)
if serialModule is None:
    raise ImportError("MS COMM Control does not appear to be installed on the PC")


def MakeDlgTemplate():
    style = (
        win32con.DS_MODALFRAME
        | win32con.WS_POPUP
        | win32con.WS_VISIBLE
        | win32con.WS_CAPTION
        | win32con.WS_SYSMENU
        | win32con.DS_SETFONT
    )
    cs = win32con.WS_CHILD | win32con.WS_VISIBLE
    dlg = [
        ["Very Basic Terminal", (0, 0, 350, 180), style, None, (8, "MS Sans Serif")],
    ]
    s = win32con.WS_TABSTOP | cs
    dlg.append(
        [
            "RICHEDIT",
            None,
            132,
            (5, 5, 340, 170),
            s
            | win32con.ES_WANTRETURN
            | win32con.ES_MULTILINE
            | win32con.ES_AUTOVSCROLL
            | win32con.WS_VSCROLL,
        ]
    )
    return dlg


####################################
#
# Serial Control
#
class MySerialControl(activex.Control, serialModule.MSComm):
    def __init__(self, parent):
        activex.Control.__init__(self)
        serialModule.MSComm.__init__(self)
        self.parent = parent

    def OnComm(self):
        self.parent.OnComm()


class TestSerDialog(dialog.Dialog):
    def __init__(self, *args):
        dialog.Dialog.__init__(*(self,) + args)
        self.olectl = None

    def OnComm(self):
        event = self.olectl.CommEvent
        if event == serialModule.OnCommConstants.comEvReceive:
            self.editwindow.ReplaceSel(self.olectl.Input)

    def OnKey(self, key):
        if self.olectl:
            self.olectl.Output = chr(key)

    def OnInitDialog(self):
        rc = dialog.Dialog.OnInitDialog(self)
        self.editwindow = self.GetDlgItem(132)
        self.editwindow.HookAllKeyStrokes(self.OnKey)

        self.olectl = MySerialControl(self)
        try:
            self.olectl.CreateControl(
                "OCX",
                win32con.WS_TABSTOP | win32con.WS_VISIBLE,
                (7, 43, 500, 300),
                self._obj_,
                131,
            )
        except win32ui.error:
            self.MessageBox("The Serial Control could not be created")
            self.olectl = None
            self.EndDialog(win32con.IDCANCEL)
        if self.olectl:
            self.olectl.Settings = SERIAL_SETTINGS
            self.olectl.CommPort = SERIAL_PORT
            self.olectl.RThreshold = 1
            try:
                self.olectl.PortOpen = 1
            except pythoncom.com_error as details:
                print(
                    "Could not open the specified serial port - %s"
                    % (details.excepinfo[2])
                )
                self.EndDialog(win32con.IDCANCEL)
        return rc

    def OnDestroy(self, msg):
        if self.olectl:
            try:
                self.olectl.PortOpen = 0
            except pythoncom.com_error as details:
                print("Error closing port - %s" % (details.excepinfo[2]))
        return dialog.Dialog.OnDestroy(self, msg)


def test():
    d = TestSerDialog(MakeDlgTemplate())
    d.DoModal()


if __name__ == "__main__":
    from . import demoutils

    if demoutils.NeedGoodGUI():
        test()



# BrandProject.py
#
# Brand a VSS project with a "build number", then optionally
# stamp DLL/EXE files with version information.

import win32api, os, string, sys
import vssutil
import bulkstamp


def BrandProject(
    vssProjectName,
    descFile,
    stampPath,
    filesToSubstitute,
    buildDesc=None,
    auto=0,
    bRebrand=0,
):
    # vssProjectName -- The name of the VSS project to brand.
    # descFile -- A test file containing descriptions of the files in the release.
    # stampPath -- The full path to where the files referenced in descFile can be found.
    path = win32api.GetFullPathName(stampPath)

    build = vssutil.MakeNewBuildNo(vssProjectName, buildDesc, auto, bRebrand)
    if build is None:
        print("Cancelled")
        return

    bulkstamp.scan(build, stampPath, descFile)
    for infile, outfile in filesToSubstitute:
        SubstituteVSSInFile(vssProjectName, infile, outfile)
    return 1


def usage(msg):
    print(msg)
    print(
        """\
%s Usage:
%s [options] vssProject descFile stampPath

Automatically brand a VSS project with an automatically incremented
build number, and stamp DLL/EXE files with the build number.

Checks that no files are checked out in the project, and finds the last
build number, and suggests the next number.

Options:
-a     - Auto increment the build number, and brand (otherwise prompt
         for the build number after looking for the previous)
-r     - Restamp the files with the existing build number.
-d     - A description for the VSS Label.
-f infile=outfile - Substitute special VSS labels in the specified text
                    file with the text extracted from VSS.
"""
        % (os.path.basename(sys.argv[0]), os.path.basename(sys.argv[0]))
    )
    sys.exit(1)


if __name__ == "__main__":
    try:
        import getopt

        opts, args = getopt.getopt(sys.argv[1:], "af:d:r")
    except getopts.error as msg:
        usage(msg)
    bAuto = bRebrand = 0
    stampFiles = []
    desc = None
    for opt, val in opts:
        if opt == "-a":
            bAuto = 1
        if opt == "-f":
            infile, outfile = string.split(val, "=", 2)
            stampFiles.append((infile, outfile))
        if opt == "-d":
            desc = val
        if opt == "-r":
            bRebrand = 1
    if len(args) < 3:
        usage("You must specify the required arguments")
    vssProjectName = "$\\" + args[0]
    descFile = args[1]
    path = args[2]
    try:
        os.stat(descFile)
    except IOError:
        usage("The description file '%s' can not be found" % (descFile))
    if not os.path.isdir(path):
        usage("The path to the files to stamp '%s' does not exist" % (path))

    BrandProject(vssProjectName, descFile, path, stampFiles, desc, bAuto, bRebrand)



# This is part of the Python test suite.
# The object is registered when you first run the test suite.
# (and hopefully unregistered once done ;-)

# Ensure the vtables in the tlb are known.
from win32com import universal
from win32com.server.exception import COMException
from win32com.client import gencache
import winerror
from win32com.client import constants
from win32com.server.util import wrap

import pythoncom

pythoncom.__future_currency__ = True
# We use the constants from the module, so must insist on a gencache.
# Otherwise, use of gencache is not necessary (tho still advised)
gencache.EnsureModule("{6BCDCB60-5605-11D0-AE5F-CADD4C000000}", 0, 1, 1)


class PyCOMTest:
    _typelib_guid_ = "{6BCDCB60-5605-11D0-AE5F-CADD4C000000}"
    _typelib_version = 1, 0
    _com_interfaces_ = ["IPyCOMTest"]
    _reg_clsid_ = "{e743d9cd-cb03-4b04-b516-11d3a81c1597}"
    _reg_progid_ = "Python.Test.PyCOMTest"

    def DoubleString(self, str):
        return str * 2

    def DoubleInOutString(self, str):
        return str * 2

    def Fire(self, nID):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def GetLastVarArgs(self):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def GetMultipleInterfaces(self, outinterface1, outinterface2):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def GetSafeArrays(self, attrs, attrs2, ints):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def GetSetDispatch(self, indisp):
        raise COMException(hresult=winerror.E_NOTIMPL)

    # Result is of type IPyCOMTest
    def GetSetInterface(self, ininterface):
        return wrap(self)

    def GetSetVariant(self, indisp):
        return indisp

    def TestByRefVariant(self, v):
        return v * 2

    def TestByRefString(self, v):
        return v * 2

    # Result is of type IPyCOMTest
    def GetSetInterfaceArray(self, ininterface):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def GetSetUnknown(self, inunk):
        raise COMException(hresult=winerror.E_NOTIMPL)

    # Result is of type ISimpleCounter
    def GetSimpleCounter(self):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def GetSimpleSafeArray(self, ints):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def GetStruct(self):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def SetIntSafeArray(self, ints):
        return len(ints)

    def SetLongLongSafeArray(self, ints):
        return len(ints)

    def SetULongLongSafeArray(self, ints):
        return len(ints)

    def SetBinSafeArray(self, buf):
        return len(buf)

    def SetVarArgs(self, *args):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def SetVariantSafeArray(self, vars):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def Start(self):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def Stop(self, nID):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def StopAll(self):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def TakeByRefDispatch(self, inout):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def TakeByRefTypedDispatch(self, inout):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def Test(self, key, inval):
        return not inval

    def Test2(self, inval):
        return inval

    def Test3(self, inval):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def Test4(self, inval):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def Test5(self, inout):
        if inout == constants.TestAttr1:
            return constants.TestAttr1_1
        elif inout == constants.TestAttr1_1:
            return constants.TestAttr1
        else:
            return -1

    def Test6(self, inval):
        return inval

    def TestInOut(self, fval, bval, lval):
        return winerror.S_OK, fval * 2, not bval, lval * 2

    def TestOptionals(self, strArg="def", sval=0, lval=1, dval=3.1400001049041748):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def TestOptionals2(self, dval, strval="", sval=1):
        raise COMException(hresult=winerror.E_NOTIMPL)

    def CheckVariantSafeArray(self, data):
        return 1

    def LongProp(self):
        return self.longval

    def SetLongProp(self, val):
        self.longval = val

    def ULongProp(self):
        return self.ulongval

    def SetULongProp(self, val):
        self.ulongval = val

    def IntProp(self):
        return self.intval

    def SetIntProp(self, val):
        self.intval = val


class PyCOMTestMI(PyCOMTest):
    _typelib_guid_ = "{6BCDCB60-5605-11D0-AE5F-CADD4C000000}"
    _typelib_version = 1, 0
    # Interfaces with a interface name, a real IID, and an IID as a string
    _com_interfaces_ = [
        "IPyCOMTest",
        pythoncom.IID_IStream,
        str(pythoncom.IID_IStorage),
    ]
    _reg_clsid_ = "{F506E9A1-FB46-4238-A597-FA4EB69787CA}"
    _reg_progid_ = "Python.Test.PyCOMTestMI"


if __name__ == "__main__":
    import win32com.server.register

    win32com.server.register.UseCommandLine(PyCOMTest)
    win32com.server.register.UseCommandLine(PyCOMTestMI)



import win32security, win32file, win32api, ntsecuritycon, win32con
from security_enums import TRUSTEE_TYPE, TRUSTEE_FORM, ACE_FLAGS, ACCESS_MODE

new_privs = (
    (
        win32security.LookupPrivilegeValue("", ntsecuritycon.SE_SECURITY_NAME),
        win32con.SE_PRIVILEGE_ENABLED,
    ),
    (
        win32security.LookupPrivilegeValue("", ntsecuritycon.SE_CREATE_PERMANENT_NAME),
        win32con.SE_PRIVILEGE_ENABLED,
    ),
    (
        win32security.LookupPrivilegeValue("", "SeEnableDelegationPrivilege"),
        win32con.SE_PRIVILEGE_ENABLED,
    ),  ##doesn't seem to be in ntsecuritycon.py ?
)

ph = win32api.GetCurrentProcess()
th = win32security.OpenProcessToken(
    ph, win32security.TOKEN_ALL_ACCESS
)  ##win32con.TOKEN_ADJUST_PRIVILEGES)
win32security.AdjustTokenPrivileges(th, 0, new_privs)

policy_handle = win32security.GetPolicyHandle("", win32security.POLICY_ALL_ACCESS)
tmp_sid = win32security.LookupAccountName("", "tmp")[0]

privs = [
    ntsecuritycon.SE_DEBUG_NAME,
    ntsecuritycon.SE_TCB_NAME,
    ntsecuritycon.SE_RESTORE_NAME,
    ntsecuritycon.SE_REMOTE_SHUTDOWN_NAME,
]
win32security.LsaAddAccountRights(policy_handle, tmp_sid, privs)

privlist = win32security.LsaEnumerateAccountRights(policy_handle, tmp_sid)
for priv in privlist:
    print(priv)

privs = [ntsecuritycon.SE_DEBUG_NAME, ntsecuritycon.SE_TCB_NAME]
win32security.LsaRemoveAccountRights(policy_handle, tmp_sid, 0, privs)

privlist = win32security.LsaEnumerateAccountRights(policy_handle, tmp_sid)
for priv in privlist:
    print(priv)

win32security.LsaClose(policy_handle)



# Imported by pywin32.pth to bootstrap the pywin32 environment in "portable"
# environments or any other case where the post-install script isn't run.
#
# In short, there's a directory installed by pywin32 named 'pywin32_system32'
# with some important DLLs which need to be found by Python when some pywin32
# modules are imported.
# If Python has `os.add_dll_directory()`, we need to call it with this path.
# Otherwise, we add this path to PATH.


try:
    import pywin32_system32
except ImportError:  # Python ≥3.6: replace ImportError with ModuleNotFoundError
    pass
else:
    import os

    # We're guaranteed only that __path__: Iterable[str]
    # https://docs.python.org/3/reference/import.html#__path__
    for path in pywin32_system32.__path__:
        if os.path.isdir(path):
            if hasattr(os, "add_dll_directory"):
                os.add_dll_directory(path)
            # This is to ensure the pywin32 path is in the beginning to find the
            # pywin32 DLLs first and prevent other PATH entries to shadow them
            elif not os.environ["PATH"].startswith(path):
                os.environ["PATH"] = os.environ["PATH"].replace(os.pathsep + path, "")
                os.environ["PATH"] = path + os.pathsep + os.environ["PATH"]
            break



# Demonstrates how to validate a password.
# See also MSKB article Q180548
#
# To use with Kerberos you need to jump through the 'targetspn' hoops.

import win32security
import sys
from sspi import ClientAuth, ServerAuth


def validate(username, password, domain=""):
    auth_info = username, domain, password
    ca = ClientAuth("NTLM", auth_info=auth_info)
    sa = ServerAuth("NTLM")

    data = err = None
    while err != 0:
        err, data = ca.authorize(data)
        err, data = sa.authorize(data)
    # If we get here without exception, we worked!


if __name__ == "__main__":
    if len(sys.argv) not in [2, 3, 4]:
        print("Usage: %s username [password [domain]]" % (__file__,))
        sys.exit(1)

    # password and domain are optional!
    password = None
    if len(sys.argv) >= 3:
        password = sys.argv[2]
    domain = ""
    if len(sys.argv) >= 4:
        domain = sys.argv[3]
    try:
        validate(sys.argv[1], password, domain)
        print("Validated OK")
    except win32security.error as details:
        hr, func, msg = details
        print("Validation failed: %s (%d)" % (msg, hr))



import win32ui
import win32con
import fontdemo
from pywin.mfc import window, docview
import commctrl

# derive from CMDIChild.  This does much work for us.


class SplitterFrame(window.MDIChildWnd):
    def __init__(self):
        # call base CreateFrame
        self.images = None
        window.MDIChildWnd.__init__(self)

    def OnCreateClient(self, cp, context):
        splitter = win32ui.CreateSplitter()
        doc = context.doc
        frame_rect = self.GetWindowRect()
        size = ((frame_rect[2] - frame_rect[0]), (frame_rect[3] - frame_rect[1]) // 2)
        sub_size = (size[0] // 2, size[1])
        splitter.CreateStatic(self, 2, 1)
        self.v1 = win32ui.CreateEditView(doc)
        self.v2 = fontdemo.FontView(doc)
        # CListControl view
        self.v3 = win32ui.CreateListView(doc)
        sub_splitter = win32ui.CreateSplitter()
        # pass "splitter" so each view knows how to get to the others
        sub_splitter.CreateStatic(splitter, 1, 2)
        sub_splitter.CreateView(self.v1, 0, 0, (sub_size))
        sub_splitter.CreateView(self.v2, 0, 1, (0, 0))  # size ignored.
        splitter.SetRowInfo(0, size[1], 0)
        splitter.CreateView(self.v3, 1, 0, (0, 0))  # size ignored.
        # Setup items in the imagelist
        self.images = win32ui.CreateImageList(32, 32, 1, 5, 5)
        self.images.Add(win32ui.GetApp().LoadIcon(win32ui.IDR_MAINFRAME))
        self.images.Add(win32ui.GetApp().LoadIcon(win32ui.IDR_PYTHONCONTYPE))
        self.images.Add(win32ui.GetApp().LoadIcon(win32ui.IDR_TEXTTYPE))
        self.v3.SetImageList(self.images, commctrl.LVSIL_NORMAL)
        self.v3.InsertItem(0, "Icon 1", 0)
        self.v3.InsertItem(0, "Icon 2", 1)
        self.v3.InsertItem(0, "Icon 3", 2)
        # 		self.v3.Arrange(commctrl.LVA_DEFAULT) Hmmm - win95 aligns left always???
        return 1

    def OnDestroy(self, msg):
        window.MDIChildWnd.OnDestroy(self, msg)
        if self.images:
            self.images.DeleteImageList()
            self.images = None

    def InitialUpdateFrame(self, doc, makeVisible):
        self.v1.ReplaceSel("Hello from Edit Window 1")
        self.v1.SetModifiedFlag(0)


class SampleTemplate(docview.DocTemplate):
    def __init__(self):
        docview.DocTemplate.__init__(
            self, win32ui.IDR_PYTHONTYPE, None, SplitterFrame, None
        )

    def InitialUpdateFrame(self, frame, doc, makeVisible):
        # 		print "frame is ", frame, frame._obj_
        # 		print "doc is ", doc, doc._obj_
        self._obj_.InitialUpdateFrame(frame, doc, makeVisible)  # call default handler.
        frame.InitialUpdateFrame(doc, makeVisible)


def demo():
    template = SampleTemplate()
    doc = template.OpenDocumentFile(None)
    doc.SetTitle("Splitter Demo")


if __name__ == "__main__":
    import demoutils

    if demoutils.NeedGoodGUI():
        demo()



import sys, string
import pythoncom

import win32api
from win32com.adsi import *

verbose_level = 0

server = ""  # Must have trailing /
local_name = win32api.GetComputerName()


def DumpRoot():
    "Dumps the root DSE"
    path = "LDAP://%srootDSE" % server
    rootdse = ADsGetObject(path)

    for item in rootdse.Get("SupportedLDAPVersion"):
        print("%s supports ldap version %s" % (path, item))

    attributes = ["CurrentTime", "defaultNamingContext"]
    for attr in attributes:
        val = rootdse.Get(attr)
        print(" %s=%s" % (attr, val))


###############################################
#
# Code taken from article titled:
# Reading attributeSchema and classSchema Objects
def _DumpClass(child):
    attrs = "Abstract lDAPDisplayName schemaIDGUID schemaNamingContext attributeSyntax oMSyntax"
    _DumpTheseAttributes(child, string.split(attrs))


def _DumpAttribute(child):
    attrs = "lDAPDisplayName schemaIDGUID adminDescription adminDisplayName rDNAttID defaultHidingValue defaultObjectCategory systemOnly defaultSecurityDescriptor"
    _DumpTheseAttributes(child, string.split(attrs))


def _DumpTheseAttributes(child, attrs):
    for attr in attrs:
        try:
            val = child.Get(attr)
        except pythoncom.com_error as details:
            continue
            # ###
            (hr, msg, exc, arg) = details
            if exc and exc[2]:
                msg = exc[2]
            val = "<Error: %s>" % (msg,)
        if verbose_level >= 2:
            print(" %s: %s=%s" % (child.Class, attr, val))


def DumpSchema():
    "Dumps the default DSE schema"
    # Bind to rootDSE to get the schemaNamingContext property.
    path = "LDAP://%srootDSE" % server
    rootdse = ADsGetObject(path)
    name = rootdse.Get("schemaNamingContext")

    # Bind to the actual schema container.
    path = "LDAP://" + server + name
    print("Binding to", path)
    ob = ADsGetObject(path)
    nclasses = nattr = nsub = nunk = 0

    # Enumerate the attribute and class objects in the schema container.
    for child in ob:
        # Find out if this is a class, attribute, or subSchema object.
        class_name = child.Class
        if class_name == "classSchema":
            _DumpClass(child)
            nclasses = nclasses + 1
        elif class_name == "attributeSchema":
            _DumpAttribute(child)
            nattr = nattr + 1
        elif class_name == "subSchema":
            nsub = nsub + 1
        else:
            print("Unknown class:", class_name)
            nunk = nunk + 1
    if verbose_level:
        print("Processed", nclasses, "classes")
        print("Processed", nattr, "attributes")
        print("Processed", nsub, "sub-schema's")
        print("Processed", nunk, "unknown types")


def _DumpObject(ob, level=0):
    prefix = "  " * level
    print("%s%s object: %s" % (prefix, ob.Class, ob.Name))
    # Do the directory object thing
    try:
        dir_ob = ADsGetObject(ob.ADsPath, IID_IDirectoryObject)
    except pythoncom.com_error:
        dir_ob = None
    if dir_ob is not None:
        info = dir_ob.GetObjectInformation()
        print("%s RDN='%s', ObjectDN='%s'" % (prefix, info.RDN, info.ObjectDN))
        # Create a list of names to fetch
        names = ["distinguishedName"]
        attrs = dir_ob.GetObjectAttributes(names)
        for attr in attrs:
            for val, typ in attr.Values:
                print("%s Attribute '%s' = %s" % (prefix, attr.AttrName, val))

    for child in ob:
        _DumpObject(child, level + 1)


def DumpAllObjects():
    "Recursively dump the entire directory!"
    path = "LDAP://%srootDSE" % server
    rootdse = ADsGetObject(path)
    name = rootdse.Get("defaultNamingContext")

    # Bind to the actual schema container.
    path = "LDAP://" + server + name
    print("Binding to", path)
    ob = ADsGetObject(path)

    # Enumerate the attribute and class objects in the schema container.
    _DumpObject(ob)


##########################################################
#
# Code taken from article:
# Example Code for Enumerating Schema Classes, Attributes, and Syntaxes

# Fill a map with VT_ datatypes, to give us better names:
vt_map = {}
for name, val in pythoncom.__dict__.items():
    if name[:3] == "VT_":
        vt_map[val] = name


def DumpSchema2():
    "Dumps the schema using an alternative technique"
    path = "LDAP://%sschema" % (server,)
    schema = ADsGetObject(path, IID_IADsContainer)
    nclass = nprop = nsyntax = 0
    for item in schema:
        item_class = string.lower(item.Class)
        if item_class == "class":
            items = []
            if item.Abstract:
                items.append("Abstract")
            if item.Auxiliary:
                items.append("Auxiliary")
            # 			if item.Structural: items.append("Structural")
            desc = string.join(items, ", ")
            import win32com.util

            iid_name = win32com.util.IIDToInterfaceName(item.PrimaryInterface)
            if verbose_level >= 2:
                print(
                    "Class: Name=%s, Flags=%s, Primary Interface=%s"
                    % (item.Name, desc, iid_name)
                )
            nclass = nclass + 1
        elif item_class == "property":
            if item.MultiValued:
                val_type = "Multi-Valued"
            else:
                val_type = "Single-Valued"
            if verbose_level >= 2:
                print("Property: Name=%s, %s" % (item.Name, val_type))
            nprop = nprop + 1
        elif item_class == "syntax":
            data_type = vt_map.get(item.OleAutoDataType, "<unknown type>")
            if verbose_level >= 2:
                print("Syntax: Name=%s, Datatype = %s" % (item.Name, data_type))
            nsyntax = nsyntax + 1
    if verbose_level >= 1:
        print("Processed", nclass, "classes")
        print("Processed", nprop, "properties")
        print("Processed", nsyntax, "syntax items")


def DumpGC():
    "Dumps the GC: object (whatever that is!)"
    ob = ADsGetObject("GC:", IID_IADsContainer)
    for sub_ob in ob:
        print("GC ob: %s (%s)" % (sub_ob.Name, sub_ob.ADsPath))


def DumpLocalUsers():
    "Dumps the local machine users"
    path = "WinNT://%s,computer" % (local_name,)
    ob = ADsGetObject(path, IID_IADsContainer)
    ob.put_Filter(["User", "Group"])
    for sub_ob in ob:
        print("User/Group: %s (%s)" % (sub_ob.Name, sub_ob.ADsPath))


def DumpLocalGroups():
    "Dumps the local machine groups"
    path = "WinNT://%s,computer" % (local_name,)
    ob = ADsGetObject(path, IID_IADsContainer)

    ob.put_Filter(["Group"])
    for sub_ob in ob:
        print("Group: %s (%s)" % (sub_ob.Name, sub_ob.ADsPath))
        # get the members
        members = sub_ob.Members()
        for member in members:
            print("  Group member: %s (%s)" % (member.Name, member.ADsPath))


def usage(tests):
    import os

    print("Usage: %s [-s server ] [-v] [Test ...]" % os.path.basename(sys.argv[0]))
    print("  -v : Verbose - print more information")
    print("  -s : server - execute the tests against the named server")
    print("where Test is one of:")
    for t in tests:
        print(t.__name__, ":", t.__doc__)
    print()
    print("If not tests are specified, all tests are run")
    sys.exit(1)


def main():
    import getopt, traceback

    tests = []
    for ob in globals().values():
        if type(ob) == type(main) and ob.__doc__:
            tests.append(ob)
    opts, args = getopt.getopt(sys.argv[1:], "s:hv")
    for opt, val in opts:
        if opt == "-s":
            if val[-1] not in "\\/":
                val = val + "/"
            global server
            server = val
        if opt == "-h":
            usage(tests)
        if opt == "-v":
            global verbose_level
            verbose_level = verbose_level + 1

    if len(args) == 0:
        print("Running all tests - use '-h' to see command-line options...")
        dotests = tests
    else:
        dotests = []
        for arg in args:
            for t in tests:
                if t.__name__ == arg:
                    dotests.append(t)
                    break
            else:
                print("Test '%s' unknown - skipping" % arg)
    if not len(dotests):
        print("Nothing to do!")
        usage(tests)
    for test in dotests:
        try:
            test()
        except:
            print("Test %s failed" % test.__name__)
            traceback.print_exc()


if __name__ == "__main__":
    main()



#
# Progress bar control example
#
# 	PyCProgressCtrl encapsulates the MFC CProgressCtrl class.  To use it,
# 	you:
#
# 	- Create the control with win32ui.CreateProgressCtrl()
# 	- Create the control window with PyCProgressCtrl.CreateWindow()
# 	- Initialize the range if you want it to be other than (0, 100) using
# 	  PyCProgressCtrl.SetRange()
# 	- Either:
# 	  - Set the step size with PyCProgressCtrl.SetStep(), and
# 	  - Increment using PyCProgressCtrl.StepIt()
# 	  or:
# 	  - Set the amount completed using PyCProgressCtrl.SetPos()
#
# Example and progress bar code courtesy of KDL Technologies, Ltd., Hong Kong SAR, China.
#

from pywin.mfc import dialog
import win32ui
import win32con


def MakeDlgTemplate():
    style = (
        win32con.DS_MODALFRAME
        | win32con.WS_POPUP
        | win32con.WS_VISIBLE
        | win32con.WS_CAPTION
        | win32con.WS_SYSMENU
        | win32con.DS_SETFONT
    )
    cs = win32con.WS_CHILD | win32con.WS_VISIBLE

    w = 215
    h = 36

    dlg = [
        [
            "Progress bar control example",
            (0, 0, w, h),
            style,
            None,
            (8, "MS Sans Serif"),
        ],
    ]

    s = win32con.WS_TABSTOP | cs

    dlg.append(
        [
            128,
            "Tick",
            win32con.IDOK,
            (10, h - 18, 50, 14),
            s | win32con.BS_DEFPUSHBUTTON,
        ]
    )

    dlg.append(
        [
            128,
            "Cancel",
            win32con.IDCANCEL,
            (w - 60, h - 18, 50, 14),
            s | win32con.BS_PUSHBUTTON,
        ]
    )

    return dlg


class TestDialog(dialog.Dialog):
    def OnInitDialog(self):
        rc = dialog.Dialog.OnInitDialog(self)
        self.pbar = win32ui.CreateProgressCtrl()
        self.pbar.CreateWindow(
            win32con.WS_CHILD | win32con.WS_VISIBLE, (10, 10, 310, 24), self, 1001
        )
        # self.pbar.SetStep (5)
        self.progress = 0
        self.pincr = 5
        return rc

    def OnOK(self):
        # NB: StepIt wraps at the end if you increment past the upper limit!
        # self.pbar.StepIt()
        self.progress = self.progress + self.pincr
        if self.progress > 100:
            self.progress = 100
        if self.progress <= 100:
            self.pbar.SetPos(self.progress)


def demo(modal=0):
    d = TestDialog(MakeDlgTemplate())
    if modal:
        d.DoModal()
    else:
        d.CreateWindow()


if __name__ == "__main__":
    demo(1)



# Creates a task-bar icon.  Run from Python.exe to see the
# messages printed.
import win32api, win32gui
import win32con, winerror
import sys, os


class MainWindow:
    def __init__(self):
        msg_TaskbarRestart = win32gui.RegisterWindowMessage("TaskbarCreated")
        message_map = {
            msg_TaskbarRestart: self.OnRestart,
            win32con.WM_DESTROY: self.OnDestroy,
            win32con.WM_COMMAND: self.OnCommand,
            win32con.WM_USER + 20: self.OnTaskbarNotify,
        }
        # Register the Window class.
        wc = win32gui.WNDCLASS()
        hinst = wc.hInstance = win32api.GetModuleHandle(None)
        wc.lpszClassName = "PythonTaskbarDemo"
        wc.style = win32con.CS_VREDRAW | win32con.CS_HREDRAW
        wc.hCursor = win32api.LoadCursor(0, win32con.IDC_ARROW)
        wc.hbrBackground = win32con.COLOR_WINDOW
        wc.lpfnWndProc = message_map  # could also specify a wndproc.

        # Don't blow up if class already registered to make testing easier
        try:
            classAtom = win32gui.RegisterClass(wc)
        except win32gui.error as err_info:
            if err_info.winerror != winerror.ERROR_CLASS_ALREADY_EXISTS:
                raise

        # Create the Window.
        style = win32con.WS_OVERLAPPED | win32con.WS_SYSMENU
        self.hwnd = win32gui.CreateWindow(
            wc.lpszClassName,
            "Taskbar Demo",
            style,
            0,
            0,
            win32con.CW_USEDEFAULT,
            win32con.CW_USEDEFAULT,
            0,
            0,
            hinst,
            None,
        )
        win32gui.UpdateWindow(self.hwnd)
        self._DoCreateIcons()

    def _DoCreateIcons(self):
        # Try and find a custom icon
        hinst = win32api.GetModuleHandle(None)
        iconPathName = os.path.abspath(
            os.path.join(os.path.split(sys.executable)[0], "pyc.ico")
        )
        if not os.path.isfile(iconPathName):
            # Look in DLLs dir, a-la py 2.5
            iconPathName = os.path.abspath(
                os.path.join(os.path.split(sys.executable)[0], "DLLs", "pyc.ico")
            )
        if not os.path.isfile(iconPathName):
            # Look in the source tree.
            iconPathName = os.path.abspath(
                os.path.join(os.path.split(sys.executable)[0], "..\\PC\\pyc.ico")
            )
        if os.path.isfile(iconPathName):
            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
            hicon = win32gui.LoadImage(
                hinst, iconPathName, win32con.IMAGE_ICON, 0, 0, icon_flags
            )
        else:
            print("Can't find a Python icon file - using default")
            hicon = win32gui.LoadIcon(0, win32con.IDI_APPLICATION)

        flags = win32gui.NIF_ICON | win32gui.NIF_MESSAGE | win32gui.NIF_TIP
        nid = (self.hwnd, 0, flags, win32con.WM_USER + 20, hicon, "Python Demo")
        try:
            win32gui.Shell_NotifyIcon(win32gui.NIM_ADD, nid)
        except win32gui.error:
            # This is common when windows is starting, and this code is hit
            # before the taskbar has been created.
            print("Failed to add the taskbar icon - is explorer running?")
            # but keep running anyway - when explorer starts, we get the
            # TaskbarCreated message.

    def OnRestart(self, hwnd, msg, wparam, lparam):
        self._DoCreateIcons()

    def OnDestroy(self, hwnd, msg, wparam, lparam):
        nid = (self.hwnd, 0)
        win32gui.Shell_NotifyIcon(win32gui.NIM_DELETE, nid)
        win32gui.PostQuitMessage(0)  # Terminate the app.

    def OnTaskbarNotify(self, hwnd, msg, wparam, lparam):
        if lparam == win32con.WM_LBUTTONUP:
            print("You clicked me.")
        elif lparam == win32con.WM_LBUTTONDBLCLK:
            print("You double-clicked me - goodbye")
            win32gui.DestroyWindow(self.hwnd)
        elif lparam == win32con.WM_RBUTTONUP:
            print("You right clicked me.")
            menu = win32gui.CreatePopupMenu()
            win32gui.AppendMenu(menu, win32con.MF_STRING, 1023, "Display Dialog")
            win32gui.AppendMenu(menu, win32con.MF_STRING, 1024, "Say Hello")
            win32gui.AppendMenu(menu, win32con.MF_STRING, 1025, "Exit program")
            pos = win32gui.GetCursorPos()
            # See http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/menus_0hdi.asp
            win32gui.SetForegroundWindow(self.hwnd)
            win32gui.TrackPopupMenu(
                menu, win32con.TPM_LEFTALIGN, pos[0], pos[1], 0, self.hwnd, None
            )
            win32gui.PostMessage(self.hwnd, win32con.WM_NULL, 0, 0)
        return 1

    def OnCommand(self, hwnd, msg, wparam, lparam):
        id = win32api.LOWORD(wparam)
        if id == 1023:
            import win32gui_dialog

            win32gui_dialog.DemoModal()
        elif id == 1024:
            print("Hello")
        elif id == 1025:
            print("Goodbye")
            win32gui.DestroyWindow(self.hwnd)
        else:
            print("Unknown command -", id)


def main():
    w = MainWindow()
    win32gui.PumpMessages()


if __name__ == "__main__":
    main()
